(function (exports, isomorphicFormData, crossFetch) {
    'use strict';

    /******************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */

    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
            function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }

    class Service {
        constructor(client) {
            this.client = client;
        }
        static flatten(data, prefix = '') {
            let output = {};
            for (const key in data) {
                let value = data[key];
                let finalKey = prefix ? `${prefix}[${key}]` : key;
                if (Array.isArray(value)) {
                    output = Object.assign(output, this.flatten(value, finalKey));
                }
                else {
                    output[finalKey] = value;
                }
            }
            return output;
        }
    }
    Service.CHUNK_SIZE = 5 * 1024 * 1024; // 5MB

    class Query {
    }
    Query.equal = (attribute, value) => Query.addQuery(attribute, "equal", value);
    Query.notEqual = (attribute, value) => Query.addQuery(attribute, "notEqual", value);
    Query.lessThan = (attribute, value) => Query.addQuery(attribute, "lessThan", value);
    Query.lessThanEqual = (attribute, value) => Query.addQuery(attribute, "lessThanEqual", value);
    Query.greaterThan = (attribute, value) => Query.addQuery(attribute, "greaterThan", value);
    Query.greaterThanEqual = (attribute, value) => Query.addQuery(attribute, "greaterThanEqual", value);
    Query.isNull = (attribute) => `isNull("${attribute}")`;
    Query.isNotNull = (attribute) => `isNotNull("${attribute}")`;
    Query.between = (attribute, start, end) => `between("${attribute}", ${Query.parseValues(start)}, ${Query.parseValues(end)})`;
    Query.startsWith = (attribute, value) => Query.addQuery(attribute, "startsWith", value);
    Query.endsWith = (attribute, value) => Query.addQuery(attribute, "endsWith", value);
    Query.select = (attributes) => `select([${attributes.map((attr) => `"${attr}"`).join(",")}])`;
    Query.search = (attribute, value) => Query.addQuery(attribute, "search", value);
    Query.orderDesc = (attribute) => `orderDesc("${attribute}")`;
    Query.orderAsc = (attribute) => `orderAsc("${attribute}")`;
    Query.cursorAfter = (documentId) => `cursorAfter("${documentId}")`;
    Query.cursorBefore = (documentId) => `cursorBefore("${documentId}")`;
    Query.limit = (limit) => `limit(${limit})`;
    Query.offset = (offset) => `offset(${offset})`;
    Query.addQuery = (attribute, method, value) => value instanceof Array
        ? `${method}("${attribute}", [${value
        .map((v) => Query.parseValues(v))
        .join(",")}])`
        : `${method}("${attribute}", [${Query.parseValues(value)}])`;
    Query.parseValues = (value) => typeof value === "string" || value instanceof String
        ? `"${value}"`
        : `${value}`;

    class AppwriteException extends Error {
        constructor(message, code = 0, type = '', response = '') {
            super(message);
            this.name = 'AppwriteException';
            this.message = message;
            this.code = code;
            this.type = type;
            this.response = response;
        }
    }
    class Client {
        constructor() {
            this.config = {
                endpoint: 'https://HOSTNAME/v1',
                endpointRealtime: '',
                project: '',
                key: '',
                jwt: '',
                locale: '',
                session: '',
                forwardedfor: '',
                forwardeduseragent: '',
            };
            this.headers = {
                'x-sdk-name': 'NAME',
                'x-sdk-platform': '',
                'x-sdk-language': 'web',
                'x-sdk-version': '0.0.0',
                'X-Appwrite-Response-Format': '1.2.0',
            };
            this.realtime = {
                socket: undefined,
                timeout: undefined,
                url: '',
                channels: new Set(),
                subscriptions: new Map(),
                subscriptionsCounter: 0,
                reconnect: true,
                reconnectAttempts: 0,
                lastMessage: undefined,
                connect: () => {
                    clearTimeout(this.realtime.timeout);
                    this.realtime.timeout = window === null || window === void 0 ? void 0 : window.setTimeout(() => {
                        this.realtime.createSocket();
                    }, 50);
                },
                getTimeout: () => {
                    switch (true) {
                        case this.realtime.reconnectAttempts < 5:
                            return 1000;
                        case this.realtime.reconnectAttempts < 15:
                            return 5000;
                        case this.realtime.reconnectAttempts < 100:
                            return 10000;
                        default:
                            return 60000;
                    }
                },
                createSocket: () => {
                    var _a, _b;
                    if (this.realtime.channels.size < 1)
                        return;
                    const channels = new URLSearchParams();
                    channels.set('project', this.config.project);
                    this.realtime.channels.forEach(channel => {
                        channels.append('channels[]', channel);
                    });
                    const url = this.config.endpointRealtime + '/realtime?' + channels.toString();
                    if (url !== this.realtime.url || // Check if URL is present
                        !this.realtime.socket || // Check if WebSocket has not been created
                        ((_a = this.realtime.socket) === null || _a === void 0 ? void 0 : _a.readyState) > WebSocket.OPEN // Check if WebSocket is CLOSING (3) or CLOSED (4)
                    ) {
                        if (this.realtime.socket &&
                            ((_b = this.realtime.socket) === null || _b === void 0 ? void 0 : _b.readyState) < WebSocket.CLOSING // Close WebSocket if it is CONNECTING (0) or OPEN (1)
                        ) {
                            this.realtime.reconnect = false;
                            this.realtime.socket.close();
                        }
                        this.realtime.url = url;
                        this.realtime.socket = new WebSocket(url);
                        this.realtime.socket.addEventListener('message', this.realtime.onMessage);
                        this.realtime.socket.addEventListener('open', _event => {
                            this.realtime.reconnectAttempts = 0;
                        });
                        this.realtime.socket.addEventListener('close', event => {
                            var _a, _b, _c;
                            if (!this.realtime.reconnect ||
                                (((_b = (_a = this.realtime) === null || _a === void 0 ? void 0 : _a.lastMessage) === null || _b === void 0 ? void 0 : _b.type) === 'error' && // Check if last message was of type error
                                    ((_c = this.realtime) === null || _c === void 0 ? void 0 : _c.lastMessage.data).code === 1008 // Check for policy violation 1008
                                )) {
                                this.realtime.reconnect = true;
                                return;
                            }
                            const timeout = this.realtime.getTimeout();
                            console.error(`Realtime got disconnected. Reconnect will be attempted in ${timeout / 1000} seconds.`, event.reason);
                            setTimeout(() => {
                                this.realtime.reconnectAttempts++;
                                this.realtime.createSocket();
                            }, timeout);
                        });
                    }
                },
                onMessage: (event) => {
                    var _a, _b;
                    try {
                        const message = JSON.parse(event.data);
                        this.realtime.lastMessage = message;
                        switch (message.type) {
                            case 'connected':
                                const cookie = JSON.parse((_a = window.localStorage.getItem('cookieFallback')) !== null && _a !== void 0 ? _a : '{}');
                                const session = cookie === null || cookie === void 0 ? void 0 : cookie[`a_session_${this.config.project}`];
                                const messageData = message.data;
                                if (session && !messageData.user) {
                                    (_b = this.realtime.socket) === null || _b === void 0 ? void 0 : _b.send(JSON.stringify({
                                        type: 'authentication',
                                        data: {
                                            session
                                        }
                                    }));
                                }
                                break;
                            case 'event':
                                let data = message.data;
                                if (data === null || data === void 0 ? void 0 : data.channels) {
                                    const isSubscribed = data.channels.some(channel => this.realtime.channels.has(channel));
                                    if (!isSubscribed)
                                        return;
                                    this.realtime.subscriptions.forEach(subscription => {
                                        if (data.channels.some(channel => subscription.channels.includes(channel))) {
                                            setTimeout(() => subscription.callback(data));
                                        }
                                    });
                                }
                                break;
                            case 'error':
                                throw message.data;
                            default:
                                break;
                        }
                    }
                    catch (e) {
                        console.error(e);
                    }
                },
                cleanUp: channels => {
                    this.realtime.channels.forEach(channel => {
                        if (channels.includes(channel)) {
                            let found = Array.from(this.realtime.subscriptions).some(([_key, subscription]) => {
                                return subscription.channels.includes(channel);
                            });
                            if (!found) {
                                this.realtime.channels.delete(channel);
                            }
                        }
                    });
                }
            };
        }
        /**
         * Set Endpoint
         *
         * Your project endpoint
         *
         * @param {string} endpoint
         *
         * @returns {this}
         */
        setEndpoint(endpoint) {
            this.config.endpoint = endpoint;
            this.config.endpointRealtime = this.config.endpointRealtime || this.config.endpoint.replace('https://', 'wss://').replace('http://', 'ws://');
            return this;
        }
        /**
         * Set Realtime Endpoint
         *
         * @param {string} endpointRealtime
         *
         * @returns {this}
         */
        setEndpointRealtime(endpointRealtime) {
            this.config.endpointRealtime = endpointRealtime;
            return this;
        }
        /**
         * Set Project
         *
         * Your project ID
         *
         * @param value string
         *
         * @return {this}
         */
        setProject(value) {
            this.headers['X-Appwrite-Project'] = value;
            this.config.project = value;
            return this;
        }
        /**
         * Set Key
         *
         * Your secret API key
         *
         * @param value string
         *
         * @return {this}
         */
        setKey(value) {
            this.headers['X-Appwrite-Key'] = value;
            this.config.key = value;
            return this;
        }
        /**
         * Set JWT
         *
         * Your secret JSON Web Token
         *
         * @param value string
         *
         * @return {this}
         */
        setJWT(value) {
            this.headers['X-Appwrite-JWT'] = value;
            this.config.jwt = value;
            return this;
        }
        /**
         * Set Locale
         *
         * @param value string
         *
         * @return {this}
         */
        setLocale(value) {
            this.headers['X-Appwrite-Locale'] = value;
            this.config.locale = value;
            return this;
        }
        /**
         * Set Session
         *
         * The user session to authenticate with
         *
         * @param value string
         *
         * @return {this}
         */
        setSession(value) {
            this.headers['X-Appwrite-Session'] = value;
            this.config.session = value;
            return this;
        }
        /**
         * Set ForwardedFor
         *
         * The IP address of the client that made the request
         *
         * @param value string
         *
         * @return {this}
         */
        setForwardedFor(value) {
            this.headers['X-Forwarded-For'] = value;
            this.config.forwardedfor = value;
            return this;
        }
        /**
         * Set ForwardedUserAgent
         *
         * The user agent string of the client that made the request
         *
         * @param value string
         *
         * @return {this}
         */
        setForwardedUserAgent(value) {
            this.headers['X-Forwarded-User-Agent'] = value;
            this.config.forwardeduseragent = value;
            return this;
        }
        /**
         * Subscribes to Appwrite events and passes you the payload in realtime.
         *
         * @param {string|string[]} channels
         * Channel to subscribe - pass a single channel as a string or multiple with an array of strings.
         *
         * Possible channels are:
         * - account
         * - collections
         * - collections.[ID]
         * - collections.[ID].documents
         * - documents
         * - documents.[ID]
         * - files
         * - files.[ID]
         * - executions
         * - executions.[ID]
         * - functions.[ID]
         * - teams
         * - teams.[ID]
         * - memberships
         * - memberships.[ID]
         * @param {(payload: RealtimeMessage) => void} callback Is called on every realtime update.
         * @returns {() => void} Unsubscribes from events.
         */
        subscribe(channels, callback) {
            let channelArray = typeof channels === 'string' ? [channels] : channels;
            channelArray.forEach(channel => this.realtime.channels.add(channel));
            const counter = this.realtime.subscriptionsCounter++;
            this.realtime.subscriptions.set(counter, {
                channels: channelArray,
                callback
            });
            this.realtime.connect();
            return () => {
                this.realtime.subscriptions.delete(counter);
                this.realtime.cleanUp(channelArray);
                this.realtime.connect();
            };
        }
        call(method, url, headers = {}, params = {}) {
            var _a, _b;
            return __awaiter(this, void 0, void 0, function* () {
                method = method.toUpperCase();
                headers = Object.assign({}, this.headers, headers);
                let options = {
                    method,
                    headers,
                    credentials: 'include'
                };
                if (typeof window !== 'undefined' && window.localStorage) {
                    headers['X-Fallback-Cookies'] = (_a = window.localStorage.getItem('cookieFallback')) !== null && _a !== void 0 ? _a : '';
                }
                if (method === 'GET') {
                    for (const [key, value] of Object.entries(Service.flatten(params))) {
                        url.searchParams.append(key, value);
                    }
                }
                else {
                    switch (headers['content-type']) {
                        case 'application/json':
                            options.body = JSON.stringify(params);
                            break;
                        case 'multipart/form-data':
                            let formData = new FormData();
                            for (const key in params) {
                                if (Array.isArray(params[key])) {
                                    params[key].forEach((value) => {
                                        formData.append(key + '[]', value);
                                    });
                                }
                                else {
                                    formData.append(key, params[key]);
                                }
                            }
                            options.body = formData;
                            delete headers['content-type'];
                            break;
                    }
                }
                try {
                    let data = null;
                    const response = yield crossFetch.fetch(url.toString(), options);
                    if ((_b = response.headers.get('content-type')) === null || _b === void 0 ? void 0 : _b.includes('application/json')) {
                        data = yield response.json();
                    }
                    else {
                        data = {
                            message: yield response.text()
                        };
                    }
                    if (400 <= response.status) {
                        throw new AppwriteException(data === null || data === void 0 ? void 0 : data.message, response.status, data === null || data === void 0 ? void 0 : data.type, data);
                    }
                    const cookieFallback = response.headers.get('X-Fallback-Cookies');
                    if (typeof window !== 'undefined' && window.localStorage && cookieFallback) {
                        window.console.warn('Appwrite is using localStorage for session management. Increase your security by adding a custom domain as your API endpoint.');
                        window.localStorage.setItem('cookieFallback', cookieFallback);
                    }
                    return data;
                }
                catch (e) {
                    if (e instanceof AppwriteException) {
                        throw e;
                    }
                    throw new AppwriteException(e.message);
                }
            });
        }
    }

    class Account extends Service {
        constructor(client) {
            super(client);
        }
        /**
         * Get account
         *
         * Get the currently logged in user.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        get() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/account';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create account
         *
         * Use this endpoint to allow a new user to register a new account in your
         * project. After the user registration completes successfully, you can use
         * the
         * [/account/verfication](https://appwrite.io/docs/references/cloud/client-web/account#createVerification)
         * route to start verifying the user email address. To allow the new user to
         * login to their new account, you need to create a new [account
         * session](https://appwrite.io/docs/references/cloud/client-web/account#createEmailSession).
         *
         * @param {string} userId
         * @param {string} email
         * @param {string} password
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        create(userId, email, password, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                const apiPath = '/account';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update email
         *
         * Update currently logged in user account email address. After changing user
         * address, the user confirmation status will get reset. A new confirmation
         * email is not sent automatically however you can use the send confirmation
         * email endpoint again to send the confirmation email. For security measures,
         * user password is required to complete this request.
         * This endpoint can also be used to convert an anonymous account to a normal
         * one, by passing an email address and a new password.
         *
         *
         * @param {string} email
         * @param {string} password
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateEmail(email, password) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                const apiPath = '/account/email';
                const payload = {};
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List Identities
         *
         * Get the list of identities for the currently logged in user.
         *
         * @param {string} queries
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listIdentities(queries) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/account/identities';
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete identity
         *
         * Delete an identity by its unique ID.
         *
         * @param {string} identityId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteIdentity(identityId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof identityId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "identityId"');
                }
                const apiPath = '/account/identities/{identityId}'.replace('{identityId}', identityId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create JWT
         *
         * Use this endpoint to create a JSON Web Token. You can use the resulting JWT
         * to authenticate on behalf of the current user when working with the
         * Appwrite server-side API and SDKs. The JWT secret is valid for 15 minutes
         * from its creation and will be invalid if the user will logout in that time
         * frame.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createJWT() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/account/jwt';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List logs
         *
         * Get the list of latest security activity logs for the currently logged in
         * user. Each log returns user IP address, location and date and time of log.
         *
         * @param {string[]} queries
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listLogs(queries) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/account/logs';
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update name
         *
         * Update currently logged in user account name.
         *
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateName(name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/account/name';
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update password
         *
         * Update currently logged in user password. For validation, user is required
         * to pass in the new password, and the old password. For users created with
         * OAuth, Team Invites and Magic URL, oldPassword is optional.
         *
         * @param {string} password
         * @param {string} oldPassword
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updatePassword(password, oldPassword) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                const apiPath = '/account/password';
                const payload = {};
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof oldPassword !== 'undefined') {
                    payload['oldPassword'] = oldPassword;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update phone
         *
         * Update the currently logged in user's phone number. After updating the
         * phone number, the phone verification status will be reset. A confirmation
         * SMS is not sent automatically, however you can use the [POST
         * /account/verification/phone](https://appwrite.io/docs/references/cloud/client-web/account#createPhoneVerification)
         * endpoint to send a confirmation SMS.
         *
         * @param {string} phone
         * @param {string} password
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updatePhone(phone, password) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof phone === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "phone"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                const apiPath = '/account/phone';
                const payload = {};
                if (typeof phone !== 'undefined') {
                    payload['phone'] = phone;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get account preferences
         *
         * Get the preferences as a key-value object for the currently logged in user.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getPrefs() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/account/prefs';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update preferences
         *
         * Update currently logged in user account preferences. The object you pass is
         * stored as is, and replaces any previous value. The maximum allowed prefs
         * size is 64kB and throws error if exceeded.
         *
         * @param {Partial<Preferences>} prefs
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updatePrefs(prefs) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof prefs === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "prefs"');
                }
                const apiPath = '/account/prefs';
                const payload = {};
                if (typeof prefs !== 'undefined') {
                    payload['prefs'] = prefs;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create password recovery
         *
         * Sends the user an email with a temporary secret key for password reset.
         * When the user clicks the confirmation link he is redirected back to your
         * app password reset URL with the secret key and email address values
         * attached to the URL query string. Use the query string params to submit a
         * request to the [PUT
         * /account/recovery](https://appwrite.io/docs/references/cloud/client-web/account#updateRecovery)
         * endpoint to complete the process. The verification link sent to the user's
         * email address is valid for 1 hour.
         *
         * @param {string} email
         * @param {string} url
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createRecovery(email, url) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                if (typeof url === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "url"');
                }
                const apiPath = '/account/recovery';
                const payload = {};
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof url !== 'undefined') {
                    payload['url'] = url;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create password recovery (confirmation)
         *
         * Use this endpoint to complete the user account password reset. Both the
         * **userId** and **secret** arguments will be passed as query parameters to
         * the redirect URL you have provided when sending your request to the [POST
         * /account/recovery](https://appwrite.io/docs/references/cloud/client-web/account#createRecovery)
         * endpoint.
         *
         * Please note that in order to avoid a [Redirect
         * Attack](https://github.com/OWASP/CheatSheetSeries/blob/master/cheatsheets/Unvalidated_Redirects_and_Forwards_Cheat_Sheet.md)
         * the only valid redirect URLs are the ones from domains you have set when
         * adding your platforms in the console interface.
         *
         * @param {string} userId
         * @param {string} secret
         * @param {string} password
         * @param {string} passwordAgain
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateRecovery(userId, secret, password, passwordAgain) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof secret === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "secret"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                if (typeof passwordAgain === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "passwordAgain"');
                }
                const apiPath = '/account/recovery';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof secret !== 'undefined') {
                    payload['secret'] = secret;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof passwordAgain !== 'undefined') {
                    payload['passwordAgain'] = passwordAgain;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List sessions
         *
         * Get the list of active sessions across different devices for the currently
         * logged in user.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listSessions() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/account/sessions';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete sessions
         *
         * Delete all sessions from the user account and remove any sessions cookies
         * from the end client.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteSessions() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/account/sessions';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create anonymous session
         *
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createAnonymousSession() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/account/sessions/anonymous';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create email session
         *
         *
         * @param {string} email
         * @param {string} password
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createEmailPasswordSession(email, password) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                const apiPath = '/account/sessions/email';
                const payload = {};
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create session (deprecated)
         *
         * Use this endpoint to complete creating the session with a custom session
         * token. Both the **userId** and **secret** arguments will be returned in the
         * response body when sending your request to the [POST
         * /users/:userId/sessions](https://appwrite.io/docs/references/cloud/server-nodejs/users#createToken)
         * endpoint.
         *
         *
         * @param {string} userId
         * @param {string} secret
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateMagicURLSession(userId, secret) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof secret === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "secret"');
                }
                const apiPath = '/account/sessions/magic-url';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof secret !== 'undefined') {
                    payload['secret'] = secret;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create OAuth2 session
         *
         * Allow the user to login to their account using the OAuth2 provider of their
         * choice. Each OAuth2 provider should be enabled from the Appwrite console
         * first. Use the success and failure arguments to provide a redirect URL's
         * back to your app when login is completed.
         *
         * If there is already an active session, the new session will be attached to
         * the logged-in account. If there are no active sessions, the server will
         * attempt to look for a user with the same email address as the email
         * received from the OAuth2 provider and attach the new session to the
         * existing user. If no matching user is found - the server will create a new
         * user.
         *
         * A user is limited to 10 active sessions at a time by default. [Learn more
         * about session
         * limits](https://appwrite.io/docs/authentication-security#limits).
         *
         *
         * @param {Provider} provider
         * @param {string} success
         * @param {string} failure
         * @param {string[]} scopes
         * @param {boolean} token
         * @throws {AppwriteException}
         * @returns {void|string}
        */
        createOAuth2Session(provider, success, failure, scopes, token) {
            if (typeof provider === 'undefined') {
                throw new AppwriteException('Missing required parameter: "provider"');
            }
            const apiPath = '/account/sessions/oauth2/{provider}'.replace('{provider}', provider);
            const payload = {};
            if (typeof success !== 'undefined') {
                payload['success'] = success;
            }
            if (typeof failure !== 'undefined') {
                payload['failure'] = failure;
            }
            if (typeof scopes !== 'undefined') {
                payload['scopes'] = scopes;
            }
            if (typeof token !== 'undefined') {
                payload['token'] = token;
            }
            const uri = new URL(this.client.config.endpoint + apiPath);
            payload['project'] = this.client.config.project;
            for (const [key, value] of Object.entries(Service.flatten(payload))) {
                uri.searchParams.append(key, value);
            }
            if (typeof window !== 'undefined' && (window === null || window === void 0 ? void 0 : window.location)) {
                window.location.href = uri.toString();
            }
            else {
                return uri;
            }
        }
        /**
         * Create session
         *
         * Use this endpoint to complete creating the session with a custom session
         * token. Both the **userId** and **secret** arguments will be returned in the
         * response body when sending your request to the [POST
         * /users/:userId/sessions](https://appwrite.io/docs/references/cloud/server-nodejs/users#createToken)
         * endpoint.
         *
         *
         * @param {string} userId
         * @param {string} secret
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createSession(userId, secret) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof secret === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "secret"');
                }
                const apiPath = '/account/sessions/token';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof secret !== 'undefined') {
                    payload['secret'] = secret;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get session
         *
         * Use this endpoint to get a logged in user's session using a Session ID.
         * Inputting 'current' will return the current session being used.
         *
         * @param {string} sessionId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getSession(sessionId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof sessionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "sessionId"');
                }
                const apiPath = '/account/sessions/{sessionId}'.replace('{sessionId}', sessionId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update OAuth session (refresh tokens)
         *
         * Access tokens have limited lifespan and expire to mitigate security risks.
         * If session was created using an OAuth provider, this route can be used to
         * "refresh" the access token.
         *
         * @param {string} sessionId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateSession(sessionId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof sessionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "sessionId"');
                }
                const apiPath = '/account/sessions/{sessionId}'.replace('{sessionId}', sessionId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete session
         *
         * Logout the user. Use 'current' as the session ID to logout on this device,
         * use a session ID to logout on another device. If you're looking to logout
         * the user on all devices, use [Delete
         * Sessions](https://appwrite.io/docs/references/cloud/client-web/account#deleteSessions)
         * instead.
         *
         * @param {string} sessionId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteSession(sessionId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof sessionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "sessionId"');
                }
                const apiPath = '/account/sessions/{sessionId}'.replace('{sessionId}', sessionId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update status
         *
         * Block the currently logged in user account. Behind the scene, the user
         * record is not deleted but permanently blocked from any access. To
         * completely delete a user, use the Users API instead.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateStatus() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/account/status';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create magic URL token
         *
         * Sends the user an email with a secret key for creating a session. If the
         * provided user ID has not been registered, a new user will be created. When
         * the user clicks the link in the email, the user is redirected back to the
         * URL you provided with the secret key and userId values attached to the URL
         * query string. Use the query string parameters to submit a request to the
         * [PUT
         * /account/sessions/magic-url](https://appwrite.io/docs/references/cloud/client-web/account#updateMagicURLSession)
         * endpoint to complete the login process. The link sent to the user's email
         * address is valid for 1 hour. If you are on a mobile device you can leave
         * the URL parameter empty, so that the login completion will be handled by
         * your Appwrite instance by default.
         *
         * A user is limited to 10 active sessions at a time by default. [Learn more
         * about session
         * limits](https://appwrite.io/docs/authentication-security#limits).
         *
         *
         * @param {string} userId
         * @param {string} email
         * @param {string} url
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createMagicURLToken(userId, email, url) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                const apiPath = '/account/tokens/magic-url';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof url !== 'undefined') {
                    payload['url'] = url;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create phone token
         *
         * Sends the user an SMS with a secret key for creating a session. If the
         * provided user ID has not be registered, a new user will be created. Use the
         * returned user ID and secret and submit a request to the [PUT
         * /account/sessions/phone](https://appwrite.io/docs/references/cloud/client-web/account#updatePhoneSession)
         * endpoint to complete the login process. The secret sent to the user's phone
         * is valid for 15 minutes.
         *
         * A user is limited to 10 active sessions at a time by default. [Learn more
         * about session
         * limits](https://appwrite.io/docs/authentication-security#limits).
         *
         * @param {string} userId
         * @param {string} phone
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createPhoneToken(userId, phone) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof phone === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "phone"');
                }
                const apiPath = '/account/tokens/phone';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof phone !== 'undefined') {
                    payload['phone'] = phone;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create email verification
         *
         * Use this endpoint to send a verification message to your user email address
         * to confirm they are the valid owners of that address. Both the **userId**
         * and **secret** arguments will be passed as query parameters to the URL you
         * have provided to be attached to the verification email. The provided URL
         * should redirect the user back to your app and allow you to complete the
         * verification process by verifying both the **userId** and **secret**
         * parameters. Learn more about how to [complete the verification
         * process](https://appwrite.io/docs/references/cloud/client-web/account#updateVerification).
         * The verification link sent to the user's email address is valid for 7 days.
         *
         * Please note that in order to avoid a [Redirect
         * Attack](https://github.com/OWASP/CheatSheetSeries/blob/master/cheatsheets/Unvalidated_Redirects_and_Forwards_Cheat_Sheet.md),
         * the only valid redirect URLs are the ones from domains you have set when
         * adding your platforms in the console interface.
         *
         *
         * @param {string} url
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createVerification(url) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof url === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "url"');
                }
                const apiPath = '/account/verification';
                const payload = {};
                if (typeof url !== 'undefined') {
                    payload['url'] = url;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create email verification (confirmation)
         *
         * Use this endpoint to complete the user email verification process. Use both
         * the **userId** and **secret** parameters that were attached to your app URL
         * to verify the user email ownership. If confirmed this route will return a
         * 200 status code.
         *
         * @param {string} userId
         * @param {string} secret
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateVerification(userId, secret) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof secret === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "secret"');
                }
                const apiPath = '/account/verification';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof secret !== 'undefined') {
                    payload['secret'] = secret;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create phone verification
         *
         * Use this endpoint to send a verification SMS to the currently logged in
         * user. This endpoint is meant for use after updating a user's phone number
         * using the
         * [accountUpdatePhone](https://appwrite.io/docs/references/cloud/client-web/account#updatePhone)
         * endpoint. Learn more about how to [complete the verification
         * process](https://appwrite.io/docs/references/cloud/client-web/account#updatePhoneVerification).
         * The verification code sent to the user's phone number is valid for 15
         * minutes.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createPhoneVerification() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/account/verification/phone';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create phone verification (confirmation)
         *
         * Use this endpoint to complete the user phone verification process. Use the
         * **userId** and **secret** that were sent to your user's phone number to
         * verify the user email ownership. If confirmed this route will return a 200
         * status code.
         *
         * @param {string} userId
         * @param {string} secret
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updatePhoneVerification(userId, secret) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof secret === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "secret"');
                }
                const apiPath = '/account/verification/phone';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof secret !== 'undefined') {
                    payload['secret'] = secret;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
    }

    class Avatars extends Service {
        constructor(client) {
            super(client);
        }
        /**
         * Get browser icon
         *
         * You can use this endpoint to show different browser icons to your users.
         * The code argument receives the browser code as it appears in your user [GET
         * /account/sessions](https://appwrite.io/docs/references/cloud/client-web/account#getSessions)
         * endpoint. Use width, height and quality arguments to change the output
         * settings.
         *
         * When one dimension is specified and the other is 0, the image is scaled
         * with preserved aspect ratio. If both dimensions are 0, the API provides an
         * image at source quality. If dimensions are not specified, the default size
         * of image returned is 100x100px.
         *
         * @param {Browser} code
         * @param {number} width
         * @param {number} height
         * @param {number} quality
         * @throws {AppwriteException}
         * @returns {URL}
        */
        getBrowser(code, width, height, quality) {
            if (typeof code === 'undefined') {
                throw new AppwriteException('Missing required parameter: "code"');
            }
            const apiPath = '/avatars/browsers/{code}'.replace('{code}', code);
            const payload = {};
            if (typeof width !== 'undefined') {
                payload['width'] = width;
            }
            if (typeof height !== 'undefined') {
                payload['height'] = height;
            }
            if (typeof quality !== 'undefined') {
                payload['quality'] = quality;
            }
            const uri = new URL(this.client.config.endpoint + apiPath);
            payload['project'] = this.client.config.project;
            payload['session'] = this.client.config.session;
            for (const [key, value] of Object.entries(Service.flatten(payload))) {
                uri.searchParams.append(key, value);
            }
            return uri;
        }
        /**
         * Get credit card icon
         *
         * The credit card endpoint will return you the icon of the credit card
         * provider you need. Use width, height and quality arguments to change the
         * output settings.
         *
         * When one dimension is specified and the other is 0, the image is scaled
         * with preserved aspect ratio. If both dimensions are 0, the API provides an
         * image at source quality. If dimensions are not specified, the default size
         * of image returned is 100x100px.
         *
         *
         * @param {CreditCard} code
         * @param {number} width
         * @param {number} height
         * @param {number} quality
         * @throws {AppwriteException}
         * @returns {URL}
        */
        getCreditCard(code, width, height, quality) {
            if (typeof code === 'undefined') {
                throw new AppwriteException('Missing required parameter: "code"');
            }
            const apiPath = '/avatars/credit-cards/{code}'.replace('{code}', code);
            const payload = {};
            if (typeof width !== 'undefined') {
                payload['width'] = width;
            }
            if (typeof height !== 'undefined') {
                payload['height'] = height;
            }
            if (typeof quality !== 'undefined') {
                payload['quality'] = quality;
            }
            const uri = new URL(this.client.config.endpoint + apiPath);
            payload['project'] = this.client.config.project;
            payload['session'] = this.client.config.session;
            for (const [key, value] of Object.entries(Service.flatten(payload))) {
                uri.searchParams.append(key, value);
            }
            return uri;
        }
        /**
         * Get favicon
         *
         * Use this endpoint to fetch the favorite icon (AKA favicon) of any remote
         * website URL.
         *
         *
         * @param {string} url
         * @throws {AppwriteException}
         * @returns {URL}
        */
        getFavicon(url) {
            if (typeof url === 'undefined') {
                throw new AppwriteException('Missing required parameter: "url"');
            }
            const apiPath = '/avatars/favicon';
            const payload = {};
            if (typeof url !== 'undefined') {
                payload['url'] = url;
            }
            const uri = new URL(this.client.config.endpoint + apiPath);
            payload['project'] = this.client.config.project;
            payload['session'] = this.client.config.session;
            for (const [key, value] of Object.entries(Service.flatten(payload))) {
                uri.searchParams.append(key, value);
            }
            return uri;
        }
        /**
         * Get country flag
         *
         * You can use this endpoint to show different country flags icons to your
         * users. The code argument receives the 2 letter country code. Use width,
         * height and quality arguments to change the output settings. Country codes
         * follow the [ISO 3166-1](https://en.wikipedia.org/wiki/ISO_3166-1) standard.
         *
         * When one dimension is specified and the other is 0, the image is scaled
         * with preserved aspect ratio. If both dimensions are 0, the API provides an
         * image at source quality. If dimensions are not specified, the default size
         * of image returned is 100x100px.
         *
         *
         * @param {Flag} code
         * @param {number} width
         * @param {number} height
         * @param {number} quality
         * @throws {AppwriteException}
         * @returns {URL}
        */
        getFlag(code, width, height, quality) {
            if (typeof code === 'undefined') {
                throw new AppwriteException('Missing required parameter: "code"');
            }
            const apiPath = '/avatars/flags/{code}'.replace('{code}', code);
            const payload = {};
            if (typeof width !== 'undefined') {
                payload['width'] = width;
            }
            if (typeof height !== 'undefined') {
                payload['height'] = height;
            }
            if (typeof quality !== 'undefined') {
                payload['quality'] = quality;
            }
            const uri = new URL(this.client.config.endpoint + apiPath);
            payload['project'] = this.client.config.project;
            payload['session'] = this.client.config.session;
            for (const [key, value] of Object.entries(Service.flatten(payload))) {
                uri.searchParams.append(key, value);
            }
            return uri;
        }
        /**
         * Get image from URL
         *
         * Use this endpoint to fetch a remote image URL and crop it to any image size
         * you want. This endpoint is very useful if you need to crop and display
         * remote images in your app or in case you want to make sure a 3rd party
         * image is properly served using a TLS protocol.
         *
         * When one dimension is specified and the other is 0, the image is scaled
         * with preserved aspect ratio. If both dimensions are 0, the API provides an
         * image at source quality. If dimensions are not specified, the default size
         * of image returned is 400x400px.
         *
         *
         * @param {string} url
         * @param {number} width
         * @param {number} height
         * @throws {AppwriteException}
         * @returns {URL}
        */
        getImage(url, width, height) {
            if (typeof url === 'undefined') {
                throw new AppwriteException('Missing required parameter: "url"');
            }
            const apiPath = '/avatars/image';
            const payload = {};
            if (typeof url !== 'undefined') {
                payload['url'] = url;
            }
            if (typeof width !== 'undefined') {
                payload['width'] = width;
            }
            if (typeof height !== 'undefined') {
                payload['height'] = height;
            }
            const uri = new URL(this.client.config.endpoint + apiPath);
            payload['project'] = this.client.config.project;
            payload['session'] = this.client.config.session;
            for (const [key, value] of Object.entries(Service.flatten(payload))) {
                uri.searchParams.append(key, value);
            }
            return uri;
        }
        /**
         * Get user initials
         *
         * Use this endpoint to show your user initials avatar icon on your website or
         * app. By default, this route will try to print your logged-in user name or
         * email initials. You can also overwrite the user name if you pass the 'name'
         * parameter. If no name is given and no user is logged, an empty avatar will
         * be returned.
         *
         * You can use the color and background params to change the avatar colors. By
         * default, a random theme will be selected. The random theme will persist for
         * the user's initials when reloading the same theme will always return for
         * the same initials.
         *
         * When one dimension is specified and the other is 0, the image is scaled
         * with preserved aspect ratio. If both dimensions are 0, the API provides an
         * image at source quality. If dimensions are not specified, the default size
         * of image returned is 100x100px.
         *
         *
         * @param {string} name
         * @param {number} width
         * @param {number} height
         * @param {string} background
         * @throws {AppwriteException}
         * @returns {URL}
        */
        getInitials(name, width, height, background) {
            const apiPath = '/avatars/initials';
            const payload = {};
            if (typeof name !== 'undefined') {
                payload['name'] = name;
            }
            if (typeof width !== 'undefined') {
                payload['width'] = width;
            }
            if (typeof height !== 'undefined') {
                payload['height'] = height;
            }
            if (typeof background !== 'undefined') {
                payload['background'] = background;
            }
            const uri = new URL(this.client.config.endpoint + apiPath);
            payload['project'] = this.client.config.project;
            payload['session'] = this.client.config.session;
            for (const [key, value] of Object.entries(Service.flatten(payload))) {
                uri.searchParams.append(key, value);
            }
            return uri;
        }
        /**
         * Get QR code
         *
         * Converts a given plain text to a QR code image. You can use the query
         * parameters to change the size and style of the resulting image.
         *
         *
         * @param {string} text
         * @param {number} size
         * @param {number} margin
         * @param {boolean} download
         * @throws {AppwriteException}
         * @returns {URL}
        */
        getQR(text, size, margin, download) {
            if (typeof text === 'undefined') {
                throw new AppwriteException('Missing required parameter: "text"');
            }
            const apiPath = '/avatars/qr';
            const payload = {};
            if (typeof text !== 'undefined') {
                payload['text'] = text;
            }
            if (typeof size !== 'undefined') {
                payload['size'] = size;
            }
            if (typeof margin !== 'undefined') {
                payload['margin'] = margin;
            }
            if (typeof download !== 'undefined') {
                payload['download'] = download;
            }
            const uri = new URL(this.client.config.endpoint + apiPath);
            payload['project'] = this.client.config.project;
            payload['session'] = this.client.config.session;
            for (const [key, value] of Object.entries(Service.flatten(payload))) {
                uri.searchParams.append(key, value);
            }
            return uri;
        }
    }

    class Databases extends Service {
        constructor(client) {
            super(client);
        }
        /**
         * List databases
         *
         * Get a list of all databases from the current Appwrite project. You can use
         * the search parameter to filter your results.
         *
         * @param {string[]} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        list(queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/databases';
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create database
         *
         * Create a new Database.
         *
         *
         * @param {string} databaseId
         * @param {string} name
         * @param {boolean} enabled
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        create(databaseId, name, enabled) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/databases';
                const payload = {};
                if (typeof databaseId !== 'undefined') {
                    payload['databaseId'] = databaseId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get database
         *
         * Get a database by its unique ID. This endpoint response returns a JSON
         * object with the database metadata.
         *
         * @param {string} databaseId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        get(databaseId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                const apiPath = '/databases/{databaseId}'.replace('{databaseId}', databaseId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update database
         *
         * Update a database by its unique ID.
         *
         * @param {string} databaseId
         * @param {string} name
         * @param {boolean} enabled
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        update(databaseId, name, enabled) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/databases/{databaseId}'.replace('{databaseId}', databaseId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete database
         *
         * Delete a database by its unique ID. Only API keys with with databases.write
         * scope can delete a database.
         *
         * @param {string} databaseId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        delete(databaseId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                const apiPath = '/databases/{databaseId}'.replace('{databaseId}', databaseId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List collections
         *
         * Get a list of all collections that belong to the provided databaseId. You
         * can use the search parameter to filter your results.
         *
         * @param {string} databaseId
         * @param {string[]} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listCollections(databaseId, queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                const apiPath = '/databases/{databaseId}/collections'.replace('{databaseId}', databaseId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create collection
         *
         * Create a new Collection. Before using this route, you should create a new
         * database resource using either a [server
         * integration](https://appwrite.io/docs/server/databases#databasesCreateCollection)
         * API or directly from your database console.
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} name
         * @param {string[]} permissions
         * @param {boolean} documentSecurity
         * @param {boolean} enabled
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createCollection(databaseId, collectionId, name, permissions, documentSecurity, enabled) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/databases/{databaseId}/collections'.replace('{databaseId}', databaseId);
                const payload = {};
                if (typeof collectionId !== 'undefined') {
                    payload['collectionId'] = collectionId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof permissions !== 'undefined') {
                    payload['permissions'] = permissions;
                }
                if (typeof documentSecurity !== 'undefined') {
                    payload['documentSecurity'] = documentSecurity;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get collection
         *
         * Get a collection by its unique ID. This endpoint response returns a JSON
         * object with the collection metadata.
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getCollection(databaseId, collectionId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update collection
         *
         * Update a collection by its unique ID.
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} name
         * @param {string[]} permissions
         * @param {boolean} documentSecurity
         * @param {boolean} enabled
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateCollection(databaseId, collectionId, name, permissions, documentSecurity, enabled) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof permissions !== 'undefined') {
                    payload['permissions'] = permissions;
                }
                if (typeof documentSecurity !== 'undefined') {
                    payload['documentSecurity'] = documentSecurity;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete collection
         *
         * Delete a collection by its unique ID. Only users with write permissions
         * have access to delete this resource.
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteCollection(databaseId, collectionId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List attributes
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string[]} queries
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listAttributes(databaseId, collectionId, queries) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create boolean attribute
         *
         * Create a boolean attribute.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {boolean} xdefault
         * @param {boolean} array
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createBooleanAttribute(databaseId, collectionId, key, required, xdefault, array) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/boolean'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                if (typeof array !== 'undefined') {
                    payload['array'] = array;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update boolean attribute
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {boolean} xdefault
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateBooleanAttribute(databaseId, collectionId, key, required, xdefault) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                if (typeof xdefault === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "xdefault"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/boolean/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create datetime attribute
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {string} xdefault
         * @param {boolean} array
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createDatetimeAttribute(databaseId, collectionId, key, required, xdefault, array) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/datetime'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                if (typeof array !== 'undefined') {
                    payload['array'] = array;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update dateTime attribute
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {string} xdefault
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateDatetimeAttribute(databaseId, collectionId, key, required, xdefault) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                if (typeof xdefault === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "xdefault"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/datetime/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create email attribute
         *
         * Create an email attribute.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {string} xdefault
         * @param {boolean} array
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createEmailAttribute(databaseId, collectionId, key, required, xdefault, array) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/email'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                if (typeof array !== 'undefined') {
                    payload['array'] = array;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update email attribute
         *
         * Update an email attribute. Changing the `default` value will not update
         * already existing documents.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {string} xdefault
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateEmailAttribute(databaseId, collectionId, key, required, xdefault) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                if (typeof xdefault === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "xdefault"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/email/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create enum attribute
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {string[]} elements
         * @param {boolean} required
         * @param {string} xdefault
         * @param {boolean} array
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createEnumAttribute(databaseId, collectionId, key, elements, required, xdefault, array) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof elements === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "elements"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/enum'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof elements !== 'undefined') {
                    payload['elements'] = elements;
                }
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                if (typeof array !== 'undefined') {
                    payload['array'] = array;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update enum attribute
         *
         * Update an enum attribute. Changing the `default` value will not update
         * already existing documents.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {string[]} elements
         * @param {boolean} required
         * @param {string} xdefault
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateEnumAttribute(databaseId, collectionId, key, elements, required, xdefault) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof elements === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "elements"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                if (typeof xdefault === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "xdefault"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/enum/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                if (typeof elements !== 'undefined') {
                    payload['elements'] = elements;
                }
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create float attribute
         *
         * Create a float attribute. Optionally, minimum and maximum values can be
         * provided.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {number} min
         * @param {number} max
         * @param {number} xdefault
         * @param {boolean} array
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createFloatAttribute(databaseId, collectionId, key, required, min, max, xdefault, array) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/float'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof min !== 'undefined') {
                    payload['min'] = min;
                }
                if (typeof max !== 'undefined') {
                    payload['max'] = max;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                if (typeof array !== 'undefined') {
                    payload['array'] = array;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update float attribute
         *
         * Update a float attribute. Changing the `default` value will not update
         * already existing documents.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {number} min
         * @param {number} max
         * @param {number} xdefault
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateFloatAttribute(databaseId, collectionId, key, required, min, max, xdefault) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                if (typeof min === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "min"');
                }
                if (typeof max === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "max"');
                }
                if (typeof xdefault === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "xdefault"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/float/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof min !== 'undefined') {
                    payload['min'] = min;
                }
                if (typeof max !== 'undefined') {
                    payload['max'] = max;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create integer attribute
         *
         * Create an integer attribute. Optionally, minimum and maximum values can be
         * provided.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {number} min
         * @param {number} max
         * @param {number} xdefault
         * @param {boolean} array
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createIntegerAttribute(databaseId, collectionId, key, required, min, max, xdefault, array) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/integer'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof min !== 'undefined') {
                    payload['min'] = min;
                }
                if (typeof max !== 'undefined') {
                    payload['max'] = max;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                if (typeof array !== 'undefined') {
                    payload['array'] = array;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update integer attribute
         *
         * Update an integer attribute. Changing the `default` value will not update
         * already existing documents.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {number} min
         * @param {number} max
         * @param {number} xdefault
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateIntegerAttribute(databaseId, collectionId, key, required, min, max, xdefault) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                if (typeof min === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "min"');
                }
                if (typeof max === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "max"');
                }
                if (typeof xdefault === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "xdefault"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/integer/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof min !== 'undefined') {
                    payload['min'] = min;
                }
                if (typeof max !== 'undefined') {
                    payload['max'] = max;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create IP address attribute
         *
         * Create IP address attribute.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {string} xdefault
         * @param {boolean} array
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createIpAttribute(databaseId, collectionId, key, required, xdefault, array) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/ip'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                if (typeof array !== 'undefined') {
                    payload['array'] = array;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update IP address attribute
         *
         * Update an ip attribute. Changing the `default` value will not update
         * already existing documents.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {string} xdefault
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateIpAttribute(databaseId, collectionId, key, required, xdefault) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                if (typeof xdefault === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "xdefault"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/ip/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create relationship attribute
         *
         * Create relationship attribute. [Learn more about relationship
         * attributes](https://appwrite.io/docs/databases-relationships#relationship-attributes).
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} relatedCollectionId
         * @param {RelationshipType} type
         * @param {boolean} twoWay
         * @param {string} key
         * @param {string} twoWayKey
         * @param {RelationMutate} onDelete
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createRelationshipAttribute(databaseId, collectionId, relatedCollectionId, type, twoWay, key, twoWayKey, onDelete) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof relatedCollectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "relatedCollectionId"');
                }
                if (typeof type === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "type"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/relationship'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof relatedCollectionId !== 'undefined') {
                    payload['relatedCollectionId'] = relatedCollectionId;
                }
                if (typeof type !== 'undefined') {
                    payload['type'] = type;
                }
                if (typeof twoWay !== 'undefined') {
                    payload['twoWay'] = twoWay;
                }
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof twoWayKey !== 'undefined') {
                    payload['twoWayKey'] = twoWayKey;
                }
                if (typeof onDelete !== 'undefined') {
                    payload['onDelete'] = onDelete;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create string attribute
         *
         * Create a string attribute.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {number} size
         * @param {boolean} required
         * @param {string} xdefault
         * @param {boolean} array
         * @param {boolean} encrypt
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createStringAttribute(databaseId, collectionId, key, size, required, xdefault, array, encrypt) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof size === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "size"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/string'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof size !== 'undefined') {
                    payload['size'] = size;
                }
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                if (typeof array !== 'undefined') {
                    payload['array'] = array;
                }
                if (typeof encrypt !== 'undefined') {
                    payload['encrypt'] = encrypt;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update string attribute
         *
         * Update a string attribute. Changing the `default` value will not update
         * already existing documents.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {string} xdefault
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateStringAttribute(databaseId, collectionId, key, required, xdefault) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                if (typeof xdefault === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "xdefault"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/string/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create URL attribute
         *
         * Create a URL attribute.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {string} xdefault
         * @param {boolean} array
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createUrlAttribute(databaseId, collectionId, key, required, xdefault, array) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/url'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                if (typeof array !== 'undefined') {
                    payload['array'] = array;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update URL attribute
         *
         * Update an url attribute. Changing the `default` value will not update
         * already existing documents.
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {boolean} required
         * @param {string} xdefault
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateUrlAttribute(databaseId, collectionId, key, required, xdefault) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof required === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "required"');
                }
                if (typeof xdefault === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "xdefault"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/url/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                if (typeof required !== 'undefined') {
                    payload['required'] = required;
                }
                if (typeof xdefault !== 'undefined') {
                    payload['default'] = xdefault;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get attribute
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getAttribute(databaseId, collectionId, key) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete attribute
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteAttribute(databaseId, collectionId, key) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update relationship attribute
         *
         * Update relationship attribute. [Learn more about relationship
         * attributes](https://appwrite.io/docs/databases-relationships#relationship-attributes).
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {RelationMutate} onDelete
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateRelationshipAttribute(databaseId, collectionId, key, onDelete) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/attributes/{key}/relationship'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                if (typeof onDelete !== 'undefined') {
                    payload['onDelete'] = onDelete;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List documents
         *
         * Get a list of all the user's documents in a given collection. You can use
         * the query params to filter your results.
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string[]} queries
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listDocuments(databaseId, collectionId, queries) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/documents'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create document
         *
         * Create a new Document. Before using this route, you should create a new
         * collection resource using either a [server
         * integration](https://appwrite.io/docs/server/databases#databasesCreateCollection)
         * API or directly from your database console.
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} documentId
         * @param {Omit<Document, keyof Models.Document>} data
         * @param {string[]} permissions
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createDocument(databaseId, collectionId, documentId, data, permissions) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof documentId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "documentId"');
                }
                if (typeof data === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "data"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/documents'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof documentId !== 'undefined') {
                    payload['documentId'] = documentId;
                }
                if (typeof data !== 'undefined') {
                    payload['data'] = data;
                }
                if (typeof permissions !== 'undefined') {
                    payload['permissions'] = permissions;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get document
         *
         * Get a document by its unique ID. This endpoint response returns a JSON
         * object with the document data.
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} documentId
         * @param {string[]} queries
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getDocument(databaseId, collectionId, documentId, queries) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof documentId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "documentId"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/documents/{documentId}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{documentId}', documentId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update document
         *
         * Update a document by its unique ID. Using the patch method you can pass
         * only specific fields that will get updated.
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} documentId
         * @param {Partial<Omit<Document, keyof Models.Document>>} data
         * @param {string[]} permissions
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateDocument(databaseId, collectionId, documentId, data, permissions) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof documentId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "documentId"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/documents/{documentId}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{documentId}', documentId);
                const payload = {};
                if (typeof data !== 'undefined') {
                    payload['data'] = data;
                }
                if (typeof permissions !== 'undefined') {
                    payload['permissions'] = permissions;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete document
         *
         * Delete a document by its unique ID.
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} documentId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteDocument(databaseId, collectionId, documentId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof documentId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "documentId"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/documents/{documentId}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{documentId}', documentId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List indexes
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string[]} queries
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listIndexes(databaseId, collectionId, queries) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/indexes'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create index
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @param {IndexType} type
         * @param {string[]} attributes
         * @param {string[]} orders
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createIndex(databaseId, collectionId, key, type, attributes, orders) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof type === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "type"');
                }
                if (typeof attributes === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "attributes"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/indexes'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId);
                const payload = {};
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof type !== 'undefined') {
                    payload['type'] = type;
                }
                if (typeof attributes !== 'undefined') {
                    payload['attributes'] = attributes;
                }
                if (typeof orders !== 'undefined') {
                    payload['orders'] = orders;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get index
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getIndex(databaseId, collectionId, key) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/indexes/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete index
         *
         *
         * @param {string} databaseId
         * @param {string} collectionId
         * @param {string} key
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteIndex(databaseId, collectionId, key) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof databaseId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "databaseId"');
                }
                if (typeof collectionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "collectionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                const apiPath = '/databases/{databaseId}/collections/{collectionId}/indexes/{key}'.replace('{databaseId}', databaseId).replace('{collectionId}', collectionId).replace('{key}', key);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
    }

    class Functions extends Service {
        constructor(client) {
            super(client);
        }
        /**
         * List functions
         *
         * Get a list of all the project's functions. You can use the query params to
         * filter your results.
         *
         * @param {string[]} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        list(queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/functions';
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create function
         *
         * Create a new function. You can pass a list of
         * [permissions](https://appwrite.io/docs/permissions) to allow different
         * project users or team with access to execute the function using the client
         * API.
         *
         * @param {string} functionId
         * @param {string} name
         * @param {Runtime} runtime
         * @param {string[]} execute
         * @param {string[]} events
         * @param {string} schedule
         * @param {number} timeout
         * @param {boolean} enabled
         * @param {boolean} logging
         * @param {string} entrypoint
         * @param {string} commands
         * @param {string} installationId
         * @param {string} providerRepositoryId
         * @param {string} providerBranch
         * @param {boolean} providerSilentMode
         * @param {string} providerRootDirectory
         * @param {string} templateRepository
         * @param {string} templateOwner
         * @param {string} templateRootDirectory
         * @param {string} templateBranch
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        create(functionId, name, runtime, execute, events, schedule, timeout, enabled, logging, entrypoint, commands, installationId, providerRepositoryId, providerBranch, providerSilentMode, providerRootDirectory, templateRepository, templateOwner, templateRootDirectory, templateBranch) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                if (typeof runtime === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "runtime"');
                }
                const apiPath = '/functions';
                const payload = {};
                if (typeof functionId !== 'undefined') {
                    payload['functionId'] = functionId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof runtime !== 'undefined') {
                    payload['runtime'] = runtime;
                }
                if (typeof execute !== 'undefined') {
                    payload['execute'] = execute;
                }
                if (typeof events !== 'undefined') {
                    payload['events'] = events;
                }
                if (typeof schedule !== 'undefined') {
                    payload['schedule'] = schedule;
                }
                if (typeof timeout !== 'undefined') {
                    payload['timeout'] = timeout;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof logging !== 'undefined') {
                    payload['logging'] = logging;
                }
                if (typeof entrypoint !== 'undefined') {
                    payload['entrypoint'] = entrypoint;
                }
                if (typeof commands !== 'undefined') {
                    payload['commands'] = commands;
                }
                if (typeof installationId !== 'undefined') {
                    payload['installationId'] = installationId;
                }
                if (typeof providerRepositoryId !== 'undefined') {
                    payload['providerRepositoryId'] = providerRepositoryId;
                }
                if (typeof providerBranch !== 'undefined') {
                    payload['providerBranch'] = providerBranch;
                }
                if (typeof providerSilentMode !== 'undefined') {
                    payload['providerSilentMode'] = providerSilentMode;
                }
                if (typeof providerRootDirectory !== 'undefined') {
                    payload['providerRootDirectory'] = providerRootDirectory;
                }
                if (typeof templateRepository !== 'undefined') {
                    payload['templateRepository'] = templateRepository;
                }
                if (typeof templateOwner !== 'undefined') {
                    payload['templateOwner'] = templateOwner;
                }
                if (typeof templateRootDirectory !== 'undefined') {
                    payload['templateRootDirectory'] = templateRootDirectory;
                }
                if (typeof templateBranch !== 'undefined') {
                    payload['templateBranch'] = templateBranch;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List runtimes
         *
         * Get a list of all runtimes that are currently active on your instance.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listRuntimes() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/functions/runtimes';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get function
         *
         * Get a function by its unique ID.
         *
         * @param {string} functionId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        get(functionId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                const apiPath = '/functions/{functionId}'.replace('{functionId}', functionId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update function
         *
         * Update function by its unique ID.
         *
         * @param {string} functionId
         * @param {string} name
         * @param {Runtime} runtime
         * @param {string[]} execute
         * @param {string[]} events
         * @param {string} schedule
         * @param {number} timeout
         * @param {boolean} enabled
         * @param {boolean} logging
         * @param {string} entrypoint
         * @param {string} commands
         * @param {string} installationId
         * @param {string} providerRepositoryId
         * @param {string} providerBranch
         * @param {boolean} providerSilentMode
         * @param {string} providerRootDirectory
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        update(functionId, name, runtime, execute, events, schedule, timeout, enabled, logging, entrypoint, commands, installationId, providerRepositoryId, providerBranch, providerSilentMode, providerRootDirectory) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/functions/{functionId}'.replace('{functionId}', functionId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof runtime !== 'undefined') {
                    payload['runtime'] = runtime;
                }
                if (typeof execute !== 'undefined') {
                    payload['execute'] = execute;
                }
                if (typeof events !== 'undefined') {
                    payload['events'] = events;
                }
                if (typeof schedule !== 'undefined') {
                    payload['schedule'] = schedule;
                }
                if (typeof timeout !== 'undefined') {
                    payload['timeout'] = timeout;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof logging !== 'undefined') {
                    payload['logging'] = logging;
                }
                if (typeof entrypoint !== 'undefined') {
                    payload['entrypoint'] = entrypoint;
                }
                if (typeof commands !== 'undefined') {
                    payload['commands'] = commands;
                }
                if (typeof installationId !== 'undefined') {
                    payload['installationId'] = installationId;
                }
                if (typeof providerRepositoryId !== 'undefined') {
                    payload['providerRepositoryId'] = providerRepositoryId;
                }
                if (typeof providerBranch !== 'undefined') {
                    payload['providerBranch'] = providerBranch;
                }
                if (typeof providerSilentMode !== 'undefined') {
                    payload['providerSilentMode'] = providerSilentMode;
                }
                if (typeof providerRootDirectory !== 'undefined') {
                    payload['providerRootDirectory'] = providerRootDirectory;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete function
         *
         * Delete a function by its unique ID.
         *
         * @param {string} functionId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        delete(functionId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                const apiPath = '/functions/{functionId}'.replace('{functionId}', functionId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List deployments
         *
         * Get a list of all the project's code deployments. You can use the query
         * params to filter your results.
         *
         * @param {string} functionId
         * @param {string[]} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listDeployments(functionId, queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                const apiPath = '/functions/{functionId}/deployments'.replace('{functionId}', functionId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create deployment
         *
         * Create a new function code deployment. Use this endpoint to upload a new
         * version of your code function. To execute your newly uploaded code, you'll
         * need to update the function's deployment to use your new deployment UID.
         *
         * This endpoint accepts a tar.gz file compressed with your code. Make sure to
         * include any dependencies your code has within the compressed file. You can
         * learn more about code packaging in the [Appwrite Cloud Functions
         * tutorial](https://appwrite.io/docs/functions).
         *
         * Use the "command" param to set the entrypoint used to execute your code.
         *
         * @param {string} functionId
         * @param {File} code
         * @param {boolean} activate
         * @param {string} entrypoint
         * @param {string} commands
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createDeployment(functionId, code, activate, entrypoint, commands, onProgress = (progress) => { }) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                if (typeof code === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "code"');
                }
                if (typeof activate === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "activate"');
                }
                const apiPath = '/functions/{functionId}/deployments'.replace('{functionId}', functionId);
                const payload = {};
                if (typeof entrypoint !== 'undefined') {
                    payload['entrypoint'] = entrypoint;
                }
                if (typeof commands !== 'undefined') {
                    payload['commands'] = commands;
                }
                if (typeof code !== 'undefined') {
                    payload['code'] = code;
                }
                if (typeof activate !== 'undefined') {
                    payload['activate'] = activate;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                if (!(code instanceof File)) {
                    throw new AppwriteException('Parameter "code" has to be a File.');
                }
                const size = code.size;
                if (size <= Service.CHUNK_SIZE) {
                    return yield this.client.call('post', uri, {
                        'content-type': 'multipart/form-data',
                    }, payload);
                }
                const apiHeaders = {
                    'content-type': 'multipart/form-data',
                };
                let offset = 0;
                let response = undefined;
                while (offset < size) {
                    let end = Math.min(offset + Service.CHUNK_SIZE - 1, size - 1);
                    apiHeaders['content-range'] = 'bytes ' + offset + '-' + end + '/' + size;
                    if (response && response.$id) {
                        apiHeaders['x-appwrite-id'] = response.$id;
                    }
                    const chunk = code.slice(offset, end + 1);
                    payload['code'] = new File([chunk], code.name);
                    response = yield this.client.call('post', uri, apiHeaders, payload);
                    if (onProgress) {
                        onProgress({
                            $id: response.$id,
                            progress: (offset / size) * 100,
                            sizeUploaded: offset,
                            chunksTotal: response.chunksTotal,
                            chunksUploaded: response.chunksUploaded
                        });
                    }
                    offset += Service.CHUNK_SIZE;
                }
                return response;
            });
        }
        /**
         * Get deployment
         *
         * Get a code deployment by its unique ID.
         *
         * @param {string} functionId
         * @param {string} deploymentId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getDeployment(functionId, deploymentId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                if (typeof deploymentId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "deploymentId"');
                }
                const apiPath = '/functions/{functionId}/deployments/{deploymentId}'.replace('{functionId}', functionId).replace('{deploymentId}', deploymentId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update function deployment
         *
         * Update the function code deployment ID using the unique function ID. Use
         * this endpoint to switch the code deployment that should be executed by the
         * execution endpoint.
         *
         * @param {string} functionId
         * @param {string} deploymentId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateDeployment(functionId, deploymentId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                if (typeof deploymentId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "deploymentId"');
                }
                const apiPath = '/functions/{functionId}/deployments/{deploymentId}'.replace('{functionId}', functionId).replace('{deploymentId}', deploymentId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete deployment
         *
         * Delete a code deployment by its unique ID.
         *
         * @param {string} functionId
         * @param {string} deploymentId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteDeployment(functionId, deploymentId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                if (typeof deploymentId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "deploymentId"');
                }
                const apiPath = '/functions/{functionId}/deployments/{deploymentId}'.replace('{functionId}', functionId).replace('{deploymentId}', deploymentId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create build
         *
         * Create a new build for an Appwrite Function deployment. This endpoint can
         * be used to retry a failed build.
         *
         * @param {string} functionId
         * @param {string} deploymentId
         * @param {string} buildId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createBuild(functionId, deploymentId, buildId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                if (typeof deploymentId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "deploymentId"');
                }
                if (typeof buildId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "buildId"');
                }
                const apiPath = '/functions/{functionId}/deployments/{deploymentId}/builds/{buildId}'.replace('{functionId}', functionId).replace('{deploymentId}', deploymentId).replace('{buildId}', buildId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Download Deployment
         *
         * Get a Deployment's contents by its unique ID. This endpoint supports range
         * requests for partial or streaming file download.
         *
         * @param {string} functionId
         * @param {string} deploymentId
         * @throws {AppwriteException}
         * @returns {URL}
        */
        downloadDeployment(functionId, deploymentId) {
            if (typeof functionId === 'undefined') {
                throw new AppwriteException('Missing required parameter: "functionId"');
            }
            if (typeof deploymentId === 'undefined') {
                throw new AppwriteException('Missing required parameter: "deploymentId"');
            }
            const apiPath = '/functions/{functionId}/deployments/{deploymentId}/download'.replace('{functionId}', functionId).replace('{deploymentId}', deploymentId);
            const payload = {};
            const uri = new URL(this.client.config.endpoint + apiPath);
            payload['project'] = this.client.config.project;
            payload['key'] = this.client.config.key;
            for (const [key, value] of Object.entries(Service.flatten(payload))) {
                uri.searchParams.append(key, value);
            }
            return uri;
        }
        /**
         * List executions
         *
         * Get a list of all the current user function execution logs. You can use the
         * query params to filter your results.
         *
         * @param {string} functionId
         * @param {string[]} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listExecutions(functionId, queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                const apiPath = '/functions/{functionId}/executions'.replace('{functionId}', functionId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create execution
         *
         * Trigger a function execution. The returned object will return you the
         * current execution status. You can ping the `Get Execution` endpoint to get
         * updates on the current execution status. Once this endpoint is called, your
         * function execution process will start asynchronously.
         *
         * @param {string} functionId
         * @param {string} body
         * @param {boolean} async
         * @param {string} xpath
         * @param {Method} method
         * @param {object} headers
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createExecution(functionId, body, async, xpath, method, headers) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                const apiPath = '/functions/{functionId}/executions'.replace('{functionId}', functionId);
                const payload = {};
                if (typeof body !== 'undefined') {
                    payload['body'] = body;
                }
                if (typeof async !== 'undefined') {
                    payload['async'] = async;
                }
                if (typeof xpath !== 'undefined') {
                    payload['path'] = xpath;
                }
                if (typeof method !== 'undefined') {
                    payload['method'] = method;
                }
                if (typeof headers !== 'undefined') {
                    payload['headers'] = headers;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get execution
         *
         * Get a function execution log by its unique ID.
         *
         * @param {string} functionId
         * @param {string} executionId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getExecution(functionId, executionId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                if (typeof executionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "executionId"');
                }
                const apiPath = '/functions/{functionId}/executions/{executionId}'.replace('{functionId}', functionId).replace('{executionId}', executionId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List variables
         *
         * Get a list of all variables of a specific function.
         *
         * @param {string} functionId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listVariables(functionId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                const apiPath = '/functions/{functionId}/variables'.replace('{functionId}', functionId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create variable
         *
         * Create a new function environment variable. These variables can be accessed
         * in the function at runtime as environment variables.
         *
         * @param {string} functionId
         * @param {string} key
         * @param {string} value
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createVariable(functionId, key, value) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                if (typeof value === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "value"');
                }
                const apiPath = '/functions/{functionId}/variables'.replace('{functionId}', functionId);
                const payload = {};
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof value !== 'undefined') {
                    payload['value'] = value;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get variable
         *
         * Get a variable by its unique ID.
         *
         * @param {string} functionId
         * @param {string} variableId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getVariable(functionId, variableId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                if (typeof variableId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "variableId"');
                }
                const apiPath = '/functions/{functionId}/variables/{variableId}'.replace('{functionId}', functionId).replace('{variableId}', variableId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update variable
         *
         * Update variable by its unique ID.
         *
         * @param {string} functionId
         * @param {string} variableId
         * @param {string} key
         * @param {string} value
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateVariable(functionId, variableId, key, value) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                if (typeof variableId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "variableId"');
                }
                if (typeof key === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "key"');
                }
                const apiPath = '/functions/{functionId}/variables/{variableId}'.replace('{functionId}', functionId).replace('{variableId}', variableId);
                const payload = {};
                if (typeof key !== 'undefined') {
                    payload['key'] = key;
                }
                if (typeof value !== 'undefined') {
                    payload['value'] = value;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete variable
         *
         * Delete a variable by its unique ID.
         *
         * @param {string} functionId
         * @param {string} variableId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteVariable(functionId, variableId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof functionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "functionId"');
                }
                if (typeof variableId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "variableId"');
                }
                const apiPath = '/functions/{functionId}/variables/{variableId}'.replace('{functionId}', functionId).replace('{variableId}', variableId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
    }

    class Graphql extends Service {
        constructor(client) {
            super(client);
        }
        /**
         * GraphQL endpoint
         *
         * Execute a GraphQL mutation.
         *
         * @param {object} query
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        query(query) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof query === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "query"');
                }
                const apiPath = '/graphql';
                const payload = {};
                if (typeof query !== 'undefined') {
                    payload['query'] = query;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'x-sdk-graphql': 'true',
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * GraphQL endpoint
         *
         * Execute a GraphQL mutation.
         *
         * @param {object} query
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        mutation(query) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof query === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "query"');
                }
                const apiPath = '/graphql/mutation';
                const payload = {};
                if (typeof query !== 'undefined') {
                    payload['query'] = query;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'x-sdk-graphql': 'true',
                    'content-type': 'application/json',
                }, payload);
            });
        }
    }

    class Health extends Service {
        constructor(client) {
            super(client);
        }
        /**
         * Get HTTP
         *
         * Check the Appwrite HTTP server is up and responsive.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        get() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get antivirus
         *
         * Check the Appwrite Antivirus server is up and connection is successful.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getAntivirus() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/anti-virus';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get cache
         *
         * Check the Appwrite in-memory cache servers are up and connection is
         * successful.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getCache() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/cache';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get DB
         *
         * Check the Appwrite database servers are up and connection is successful.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getDB() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/db';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get pubsub
         *
         * Check the Appwrite pub-sub servers are up and connection is successful.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getPubSub() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/pubsub';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get queue
         *
         * Check the Appwrite queue messaging servers are up and connection is
         * successful.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getQueue() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/queue';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get builds queue
         *
         * Get the number of builds that are waiting to be processed in the Appwrite
         * internal queue server.
         *
         * @param {number} threshold
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getQueueBuilds(threshold) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/queue/builds';
                const payload = {};
                if (typeof threshold !== 'undefined') {
                    payload['threshold'] = threshold;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get certificates queue
         *
         * Get the number of certificates that are waiting to be issued against
         * [Letsencrypt](https://letsencrypt.org/) in the Appwrite internal queue
         * server.
         *
         * @param {number} threshold
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getQueueCertificates(threshold) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/queue/certificates';
                const payload = {};
                if (typeof threshold !== 'undefined') {
                    payload['threshold'] = threshold;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get databases queue
         *
         * Get the number of database changes that are waiting to be processed in the
         * Appwrite internal queue server.
         *
         * @param {string} name
         * @param {number} threshold
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getQueueDatabases(name, threshold) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/queue/databases';
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof threshold !== 'undefined') {
                    payload['threshold'] = threshold;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get deletes queue
         *
         * Get the number of background destructive changes that are waiting to be
         * processed in the Appwrite internal queue server.
         *
         * @param {number} threshold
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getQueueDeletes(threshold) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/queue/deletes';
                const payload = {};
                if (typeof threshold !== 'undefined') {
                    payload['threshold'] = threshold;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get functions queue
         *
         *
         * @param {number} threshold
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getQueueFunctions(threshold) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/queue/functions';
                const payload = {};
                if (typeof threshold !== 'undefined') {
                    payload['threshold'] = threshold;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get logs queue
         *
         * Get the number of logs that are waiting to be processed in the Appwrite
         * internal queue server.
         *
         * @param {number} threshold
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getQueueLogs(threshold) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/queue/logs';
                const payload = {};
                if (typeof threshold !== 'undefined') {
                    payload['threshold'] = threshold;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get mails queue
         *
         * Get the number of mails that are waiting to be processed in the Appwrite
         * internal queue server.
         *
         * @param {number} threshold
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getQueueMails(threshold) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/queue/mails';
                const payload = {};
                if (typeof threshold !== 'undefined') {
                    payload['threshold'] = threshold;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get messaging queue
         *
         * Get the number of messages that are waiting to be processed in the Appwrite
         * internal queue server.
         *
         * @param {number} threshold
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getQueueMessaging(threshold) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/queue/messaging';
                const payload = {};
                if (typeof threshold !== 'undefined') {
                    payload['threshold'] = threshold;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get migrations queue
         *
         * Get the number of migrations that are waiting to be processed in the
         * Appwrite internal queue server.
         *
         * @param {number} threshold
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getQueueMigrations(threshold) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/queue/migrations';
                const payload = {};
                if (typeof threshold !== 'undefined') {
                    payload['threshold'] = threshold;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get webhooks queue
         *
         * Get the number of webhooks that are waiting to be processed in the Appwrite
         * internal queue server.
         *
         * @param {number} threshold
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getQueueWebhooks(threshold) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/queue/webhooks';
                const payload = {};
                if (typeof threshold !== 'undefined') {
                    payload['threshold'] = threshold;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get local storage
         *
         * Check the Appwrite local storage device is up and connection is successful.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getStorageLocal() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/storage/local';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get time
         *
         * Check the Appwrite server time is synced with Google remote NTP server. We
         * use this technology to smoothly handle leap seconds with no disruptive
         * events. The [Network Time
         * Protocol](https://en.wikipedia.org/wiki/Network_Time_Protocol) (NTP) is
         * used by hundreds of millions of computers and devices to synchronize their
         * clocks over the Internet. If your computer sets its own clock, it likely
         * uses NTP.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getTime() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/health/time';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
    }

    class Locale extends Service {
        constructor(client) {
            super(client);
        }
        /**
         * Get user locale
         *
         * Get the current user location based on IP. Returns an object with user
         * country code, country name, continent name, continent code, ip address and
         * suggested currency. You can use the locale header to get the data in a
         * supported language.
         *
         * ([IP Geolocation by DB-IP](https://db-ip.com))
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        get() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/locale';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List Locale Codes
         *
         * List of all locale codes in [ISO
         * 639-1](https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes).
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listCodes() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/locale/codes';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List continents
         *
         * List of all continents. You can use the locale header to get the data in a
         * supported language.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listContinents() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/locale/continents';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List countries
         *
         * List of all countries. You can use the locale header to get the data in a
         * supported language.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listCountries() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/locale/countries';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List EU countries
         *
         * List of all countries that are currently members of the EU. You can use the
         * locale header to get the data in a supported language.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listCountriesEU() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/locale/countries/eu';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List countries phone codes
         *
         * List of all countries phone codes. You can use the locale header to get the
         * data in a supported language.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listCountriesPhones() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/locale/countries/phones';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List currencies
         *
         * List of all currencies, including currency symbol, name, plural, and
         * decimal digits for all major and minor currencies. You can use the locale
         * header to get the data in a supported language.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listCurrencies() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/locale/currencies';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List languages
         *
         * List of all languages classified by ISO 639-1 including 2-letter code, name
         * in English, and name in the respective language.
         *
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listLanguages() {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/locale/languages';
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
    }

    class Messaging extends Service {
        constructor(client) {
            super(client);
        }
        /**
         * List messages
         *
         *
         * @param {string} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listMessages(queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/messaging/messages';
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create an email.
         *
         *
         * @param {string} messageId
         * @param {string} subject
         * @param {string} content
         * @param {string[]} topics
         * @param {string[]} users
         * @param {string[]} targets
         * @param {string[]} cc
         * @param {string[]} bcc
         * @param {string} description
         * @param {Status} status
         * @param {boolean} html
         * @param {string} scheduledAt
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createEmailMessage(messageId, subject, content, topics, users, targets, cc, bcc, description, status, html, scheduledAt) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof messageId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "messageId"');
                }
                if (typeof subject === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "subject"');
                }
                if (typeof content === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "content"');
                }
                const apiPath = '/messaging/messages/email';
                const payload = {};
                if (typeof messageId !== 'undefined') {
                    payload['messageId'] = messageId;
                }
                if (typeof subject !== 'undefined') {
                    payload['subject'] = subject;
                }
                if (typeof content !== 'undefined') {
                    payload['content'] = content;
                }
                if (typeof topics !== 'undefined') {
                    payload['topics'] = topics;
                }
                if (typeof users !== 'undefined') {
                    payload['users'] = users;
                }
                if (typeof targets !== 'undefined') {
                    payload['targets'] = targets;
                }
                if (typeof cc !== 'undefined') {
                    payload['cc'] = cc;
                }
                if (typeof bcc !== 'undefined') {
                    payload['bcc'] = bcc;
                }
                if (typeof description !== 'undefined') {
                    payload['description'] = description;
                }
                if (typeof status !== 'undefined') {
                    payload['status'] = status;
                }
                if (typeof html !== 'undefined') {
                    payload['html'] = html;
                }
                if (typeof scheduledAt !== 'undefined') {
                    payload['scheduledAt'] = scheduledAt;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update an email.
         *
         *
         * @param {string} messageId
         * @param {string[]} topics
         * @param {string[]} users
         * @param {string[]} targets
         * @param {string} subject
         * @param {string} description
         * @param {string} content
         * @param {Status} status
         * @param {boolean} html
         * @param {string[]} cc
         * @param {string[]} bcc
         * @param {string} scheduledAt
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateEmail(messageId, topics, users, targets, subject, description, content, status, html, cc, bcc, scheduledAt) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof messageId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "messageId"');
                }
                const apiPath = '/messaging/messages/email/{messageId}'.replace('{messageId}', messageId);
                const payload = {};
                if (typeof topics !== 'undefined') {
                    payload['topics'] = topics;
                }
                if (typeof users !== 'undefined') {
                    payload['users'] = users;
                }
                if (typeof targets !== 'undefined') {
                    payload['targets'] = targets;
                }
                if (typeof subject !== 'undefined') {
                    payload['subject'] = subject;
                }
                if (typeof description !== 'undefined') {
                    payload['description'] = description;
                }
                if (typeof content !== 'undefined') {
                    payload['content'] = content;
                }
                if (typeof status !== 'undefined') {
                    payload['status'] = status;
                }
                if (typeof html !== 'undefined') {
                    payload['html'] = html;
                }
                if (typeof cc !== 'undefined') {
                    payload['cc'] = cc;
                }
                if (typeof bcc !== 'undefined') {
                    payload['bcc'] = bcc;
                }
                if (typeof scheduledAt !== 'undefined') {
                    payload['scheduledAt'] = scheduledAt;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create a push notification.
         *
         *
         * @param {string} messageId
         * @param {string} title
         * @param {string} body
         * @param {string[]} topics
         * @param {string[]} users
         * @param {string[]} targets
         * @param {string} description
         * @param {object} data
         * @param {string} action
         * @param {string} icon
         * @param {string} sound
         * @param {string} color
         * @param {string} tag
         * @param {string} badge
         * @param {Status} status
         * @param {string} scheduledAt
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createPushMessage(messageId, title, body, topics, users, targets, description, data, action, icon, sound, color, tag, badge, status, scheduledAt) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof messageId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "messageId"');
                }
                if (typeof title === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "title"');
                }
                if (typeof body === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "body"');
                }
                const apiPath = '/messaging/messages/push';
                const payload = {};
                if (typeof messageId !== 'undefined') {
                    payload['messageId'] = messageId;
                }
                if (typeof title !== 'undefined') {
                    payload['title'] = title;
                }
                if (typeof body !== 'undefined') {
                    payload['body'] = body;
                }
                if (typeof topics !== 'undefined') {
                    payload['topics'] = topics;
                }
                if (typeof users !== 'undefined') {
                    payload['users'] = users;
                }
                if (typeof targets !== 'undefined') {
                    payload['targets'] = targets;
                }
                if (typeof description !== 'undefined') {
                    payload['description'] = description;
                }
                if (typeof data !== 'undefined') {
                    payload['data'] = data;
                }
                if (typeof action !== 'undefined') {
                    payload['action'] = action;
                }
                if (typeof icon !== 'undefined') {
                    payload['icon'] = icon;
                }
                if (typeof sound !== 'undefined') {
                    payload['sound'] = sound;
                }
                if (typeof color !== 'undefined') {
                    payload['color'] = color;
                }
                if (typeof tag !== 'undefined') {
                    payload['tag'] = tag;
                }
                if (typeof badge !== 'undefined') {
                    payload['badge'] = badge;
                }
                if (typeof status !== 'undefined') {
                    payload['status'] = status;
                }
                if (typeof scheduledAt !== 'undefined') {
                    payload['scheduledAt'] = scheduledAt;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update a push notification.
         *
         *
         * @param {string} messageId
         * @param {string[]} topics
         * @param {string[]} users
         * @param {string[]} targets
         * @param {string} description
         * @param {string} title
         * @param {string} body
         * @param {object} data
         * @param {string} action
         * @param {string} icon
         * @param {string} sound
         * @param {string} color
         * @param {string} tag
         * @param {number} badge
         * @param {Status} status
         * @param {string} scheduledAt
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updatePushNotification(messageId, topics, users, targets, description, title, body, data, action, icon, sound, color, tag, badge, status, scheduledAt) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof messageId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "messageId"');
                }
                const apiPath = '/messaging/messages/push/{messageId}'.replace('{messageId}', messageId);
                const payload = {};
                if (typeof topics !== 'undefined') {
                    payload['topics'] = topics;
                }
                if (typeof users !== 'undefined') {
                    payload['users'] = users;
                }
                if (typeof targets !== 'undefined') {
                    payload['targets'] = targets;
                }
                if (typeof description !== 'undefined') {
                    payload['description'] = description;
                }
                if (typeof title !== 'undefined') {
                    payload['title'] = title;
                }
                if (typeof body !== 'undefined') {
                    payload['body'] = body;
                }
                if (typeof data !== 'undefined') {
                    payload['data'] = data;
                }
                if (typeof action !== 'undefined') {
                    payload['action'] = action;
                }
                if (typeof icon !== 'undefined') {
                    payload['icon'] = icon;
                }
                if (typeof sound !== 'undefined') {
                    payload['sound'] = sound;
                }
                if (typeof color !== 'undefined') {
                    payload['color'] = color;
                }
                if (typeof tag !== 'undefined') {
                    payload['tag'] = tag;
                }
                if (typeof badge !== 'undefined') {
                    payload['badge'] = badge;
                }
                if (typeof status !== 'undefined') {
                    payload['status'] = status;
                }
                if (typeof scheduledAt !== 'undefined') {
                    payload['scheduledAt'] = scheduledAt;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create an SMS.
         *
         *
         * @param {string} messageId
         * @param {string} content
         * @param {string[]} topics
         * @param {string[]} users
         * @param {string[]} targets
         * @param {string} description
         * @param {Status} status
         * @param {string} scheduledAt
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createSMSMessage(messageId, content, topics, users, targets, description, status, scheduledAt) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof messageId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "messageId"');
                }
                if (typeof content === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "content"');
                }
                const apiPath = '/messaging/messages/sms';
                const payload = {};
                if (typeof messageId !== 'undefined') {
                    payload['messageId'] = messageId;
                }
                if (typeof content !== 'undefined') {
                    payload['content'] = content;
                }
                if (typeof topics !== 'undefined') {
                    payload['topics'] = topics;
                }
                if (typeof users !== 'undefined') {
                    payload['users'] = users;
                }
                if (typeof targets !== 'undefined') {
                    payload['targets'] = targets;
                }
                if (typeof description !== 'undefined') {
                    payload['description'] = description;
                }
                if (typeof status !== 'undefined') {
                    payload['status'] = status;
                }
                if (typeof scheduledAt !== 'undefined') {
                    payload['scheduledAt'] = scheduledAt;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update an SMS.
         *
         *
         * @param {string} messageId
         * @param {string[]} topics
         * @param {string[]} users
         * @param {string[]} targets
         * @param {string} description
         * @param {string} content
         * @param {Status} status
         * @param {string} scheduledAt
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateSMS(messageId, topics, users, targets, description, content, status, scheduledAt) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof messageId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "messageId"');
                }
                const apiPath = '/messaging/messages/sms/{messageId}'.replace('{messageId}', messageId);
                const payload = {};
                if (typeof topics !== 'undefined') {
                    payload['topics'] = topics;
                }
                if (typeof users !== 'undefined') {
                    payload['users'] = users;
                }
                if (typeof targets !== 'undefined') {
                    payload['targets'] = targets;
                }
                if (typeof description !== 'undefined') {
                    payload['description'] = description;
                }
                if (typeof content !== 'undefined') {
                    payload['content'] = content;
                }
                if (typeof status !== 'undefined') {
                    payload['status'] = status;
                }
                if (typeof scheduledAt !== 'undefined') {
                    payload['scheduledAt'] = scheduledAt;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get a message
         *
         *
         * @param {string} messageId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getMessage(messageId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof messageId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "messageId"');
                }
                const apiPath = '/messaging/messages/{messageId}'.replace('{messageId}', messageId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List message logs
         *
         *
         * @param {string} messageId
         * @param {string[]} queries
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listMessageLogs(messageId, queries) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof messageId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "messageId"');
                }
                const apiPath = '/messaging/messages/{messageId}/logs'.replace('{messageId}', messageId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List providers
         *
         *
         * @param {string} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listProviders(queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/messaging/providers';
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create APNS provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {string} authKey
         * @param {string} authKeyId
         * @param {string} teamId
         * @param {string} bundleId
         * @param {string} endpoint
         * @param {boolean} enabled
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createApnsProvider(providerId, name, authKey, authKeyId, teamId, bundleId, endpoint, enabled) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/messaging/providers/apns';
                const payload = {};
                if (typeof providerId !== 'undefined') {
                    payload['providerId'] = providerId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof authKey !== 'undefined') {
                    payload['authKey'] = authKey;
                }
                if (typeof authKeyId !== 'undefined') {
                    payload['authKeyId'] = authKeyId;
                }
                if (typeof teamId !== 'undefined') {
                    payload['teamId'] = teamId;
                }
                if (typeof bundleId !== 'undefined') {
                    payload['bundleId'] = bundleId;
                }
                if (typeof endpoint !== 'undefined') {
                    payload['endpoint'] = endpoint;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update APNS provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {boolean} enabled
         * @param {string} authKey
         * @param {string} authKeyId
         * @param {string} teamId
         * @param {string} bundleId
         * @param {string} endpoint
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateApnsProvider(providerId, name, enabled, authKey, authKeyId, teamId, bundleId, endpoint) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                const apiPath = '/messaging/providers/apns/{providerId}'.replace('{providerId}', providerId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof authKey !== 'undefined') {
                    payload['authKey'] = authKey;
                }
                if (typeof authKeyId !== 'undefined') {
                    payload['authKeyId'] = authKeyId;
                }
                if (typeof teamId !== 'undefined') {
                    payload['teamId'] = teamId;
                }
                if (typeof bundleId !== 'undefined') {
                    payload['bundleId'] = bundleId;
                }
                if (typeof endpoint !== 'undefined') {
                    payload['endpoint'] = endpoint;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create FCM provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {object} serviceAccountJSON
         * @param {boolean} enabled
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createFcmProvider(providerId, name, serviceAccountJSON, enabled) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/messaging/providers/fcm';
                const payload = {};
                if (typeof providerId !== 'undefined') {
                    payload['providerId'] = providerId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof serviceAccountJSON !== 'undefined') {
                    payload['serviceAccountJSON'] = serviceAccountJSON;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update FCM provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {boolean} enabled
         * @param {object} serviceAccountJSON
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateFcmProvider(providerId, name, enabled, serviceAccountJSON) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                const apiPath = '/messaging/providers/fcm/{providerId}'.replace('{providerId}', providerId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof serviceAccountJSON !== 'undefined') {
                    payload['serviceAccountJSON'] = serviceAccountJSON;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create Mailgun provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {string} apiKey
         * @param {string} domain
         * @param {boolean} isEuRegion
         * @param {boolean} enabled
         * @param {string} fromName
         * @param {string} fromEmail
         * @param {string} replyToName
         * @param {string} replyToEmail
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createMailgunProvider(providerId, name, apiKey, domain, isEuRegion, enabled, fromName, fromEmail, replyToName, replyToEmail) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/messaging/providers/mailgun';
                const payload = {};
                if (typeof providerId !== 'undefined') {
                    payload['providerId'] = providerId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof apiKey !== 'undefined') {
                    payload['apiKey'] = apiKey;
                }
                if (typeof domain !== 'undefined') {
                    payload['domain'] = domain;
                }
                if (typeof isEuRegion !== 'undefined') {
                    payload['isEuRegion'] = isEuRegion;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof fromName !== 'undefined') {
                    payload['fromName'] = fromName;
                }
                if (typeof fromEmail !== 'undefined') {
                    payload['fromEmail'] = fromEmail;
                }
                if (typeof replyToName !== 'undefined') {
                    payload['replyToName'] = replyToName;
                }
                if (typeof replyToEmail !== 'undefined') {
                    payload['replyToEmail'] = replyToEmail;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update Mailgun provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {string} apiKey
         * @param {string} domain
         * @param {boolean} isEuRegion
         * @param {boolean} enabled
         * @param {string} fromName
         * @param {string} fromEmail
         * @param {string} replyToName
         * @param {string} replyToEmail
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateMailgunProvider(providerId, name, apiKey, domain, isEuRegion, enabled, fromName, fromEmail, replyToName, replyToEmail) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                const apiPath = '/messaging/providers/mailgun/{providerId}'.replace('{providerId}', providerId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof apiKey !== 'undefined') {
                    payload['apiKey'] = apiKey;
                }
                if (typeof domain !== 'undefined') {
                    payload['domain'] = domain;
                }
                if (typeof isEuRegion !== 'undefined') {
                    payload['isEuRegion'] = isEuRegion;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof fromName !== 'undefined') {
                    payload['fromName'] = fromName;
                }
                if (typeof fromEmail !== 'undefined') {
                    payload['fromEmail'] = fromEmail;
                }
                if (typeof replyToName !== 'undefined') {
                    payload['replyToName'] = replyToName;
                }
                if (typeof replyToEmail !== 'undefined') {
                    payload['replyToEmail'] = replyToEmail;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create Msg91 provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {string} from
         * @param {string} senderId
         * @param {string} authKey
         * @param {boolean} enabled
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createMsg91Provider(providerId, name, from, senderId, authKey, enabled) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/messaging/providers/msg91';
                const payload = {};
                if (typeof providerId !== 'undefined') {
                    payload['providerId'] = providerId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof from !== 'undefined') {
                    payload['from'] = from;
                }
                if (typeof senderId !== 'undefined') {
                    payload['senderId'] = senderId;
                }
                if (typeof authKey !== 'undefined') {
                    payload['authKey'] = authKey;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update Msg91 provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {boolean} enabled
         * @param {string} senderId
         * @param {string} authKey
         * @param {string} from
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateMsg91Provider(providerId, name, enabled, senderId, authKey, from) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                const apiPath = '/messaging/providers/msg91/{providerId}'.replace('{providerId}', providerId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof senderId !== 'undefined') {
                    payload['senderId'] = senderId;
                }
                if (typeof authKey !== 'undefined') {
                    payload['authKey'] = authKey;
                }
                if (typeof from !== 'undefined') {
                    payload['from'] = from;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create Sendgrid provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {string} apiKey
         * @param {boolean} enabled
         * @param {string} fromName
         * @param {string} fromEmail
         * @param {string} replyToName
         * @param {string} replyToEmail
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createSendgridProvider(providerId, name, apiKey, enabled, fromName, fromEmail, replyToName, replyToEmail) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/messaging/providers/sendgrid';
                const payload = {};
                if (typeof providerId !== 'undefined') {
                    payload['providerId'] = providerId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof apiKey !== 'undefined') {
                    payload['apiKey'] = apiKey;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof fromName !== 'undefined') {
                    payload['fromName'] = fromName;
                }
                if (typeof fromEmail !== 'undefined') {
                    payload['fromEmail'] = fromEmail;
                }
                if (typeof replyToName !== 'undefined') {
                    payload['replyToName'] = replyToName;
                }
                if (typeof replyToEmail !== 'undefined') {
                    payload['replyToEmail'] = replyToEmail;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update Sendgrid provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {boolean} enabled
         * @param {string} apiKey
         * @param {string} fromName
         * @param {string} fromEmail
         * @param {string} replyToName
         * @param {string} replyToEmail
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateSendgridProvider(providerId, name, enabled, apiKey, fromName, fromEmail, replyToName, replyToEmail) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                const apiPath = '/messaging/providers/sendgrid/{providerId}'.replace('{providerId}', providerId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof apiKey !== 'undefined') {
                    payload['apiKey'] = apiKey;
                }
                if (typeof fromName !== 'undefined') {
                    payload['fromName'] = fromName;
                }
                if (typeof fromEmail !== 'undefined') {
                    payload['fromEmail'] = fromEmail;
                }
                if (typeof replyToName !== 'undefined') {
                    payload['replyToName'] = replyToName;
                }
                if (typeof replyToEmail !== 'undefined') {
                    payload['replyToEmail'] = replyToEmail;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create Telesign provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {string} from
         * @param {string} username
         * @param {string} password
         * @param {boolean} enabled
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createTelesignProvider(providerId, name, from, username, password, enabled) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/messaging/providers/telesign';
                const payload = {};
                if (typeof providerId !== 'undefined') {
                    payload['providerId'] = providerId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof from !== 'undefined') {
                    payload['from'] = from;
                }
                if (typeof username !== 'undefined') {
                    payload['username'] = username;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update Telesign provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {boolean} enabled
         * @param {string} username
         * @param {string} password
         * @param {string} from
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateTelesignProvider(providerId, name, enabled, username, password, from) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                const apiPath = '/messaging/providers/telesign/{providerId}'.replace('{providerId}', providerId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof username !== 'undefined') {
                    payload['username'] = username;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof from !== 'undefined') {
                    payload['from'] = from;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create Textmagic provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {string} from
         * @param {string} username
         * @param {string} apiKey
         * @param {boolean} enabled
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createTextmagicProvider(providerId, name, from, username, apiKey, enabled) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/messaging/providers/textmagic';
                const payload = {};
                if (typeof providerId !== 'undefined') {
                    payload['providerId'] = providerId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof from !== 'undefined') {
                    payload['from'] = from;
                }
                if (typeof username !== 'undefined') {
                    payload['username'] = username;
                }
                if (typeof apiKey !== 'undefined') {
                    payload['apiKey'] = apiKey;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update Textmagic provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {boolean} enabled
         * @param {string} username
         * @param {string} apiKey
         * @param {string} from
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateTextmagicProvider(providerId, name, enabled, username, apiKey, from) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                const apiPath = '/messaging/providers/textmagic/{providerId}'.replace('{providerId}', providerId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof username !== 'undefined') {
                    payload['username'] = username;
                }
                if (typeof apiKey !== 'undefined') {
                    payload['apiKey'] = apiKey;
                }
                if (typeof from !== 'undefined') {
                    payload['from'] = from;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create Twilio provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {string} from
         * @param {string} accountSid
         * @param {string} authToken
         * @param {boolean} enabled
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createTwilioProvider(providerId, name, from, accountSid, authToken, enabled) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/messaging/providers/twilio';
                const payload = {};
                if (typeof providerId !== 'undefined') {
                    payload['providerId'] = providerId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof from !== 'undefined') {
                    payload['from'] = from;
                }
                if (typeof accountSid !== 'undefined') {
                    payload['accountSid'] = accountSid;
                }
                if (typeof authToken !== 'undefined') {
                    payload['authToken'] = authToken;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update Twilio provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {boolean} enabled
         * @param {string} accountSid
         * @param {string} authToken
         * @param {string} from
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateTwilioProvider(providerId, name, enabled, accountSid, authToken, from) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                const apiPath = '/messaging/providers/twilio/{providerId}'.replace('{providerId}', providerId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof accountSid !== 'undefined') {
                    payload['accountSid'] = accountSid;
                }
                if (typeof authToken !== 'undefined') {
                    payload['authToken'] = authToken;
                }
                if (typeof from !== 'undefined') {
                    payload['from'] = from;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create Vonage provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {string} from
         * @param {string} apiKey
         * @param {string} apiSecret
         * @param {boolean} enabled
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createVonageProvider(providerId, name, from, apiKey, apiSecret, enabled) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/messaging/providers/vonage';
                const payload = {};
                if (typeof providerId !== 'undefined') {
                    payload['providerId'] = providerId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof from !== 'undefined') {
                    payload['from'] = from;
                }
                if (typeof apiKey !== 'undefined') {
                    payload['apiKey'] = apiKey;
                }
                if (typeof apiSecret !== 'undefined') {
                    payload['apiSecret'] = apiSecret;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update Vonage provider
         *
         *
         * @param {string} providerId
         * @param {string} name
         * @param {boolean} enabled
         * @param {string} apiKey
         * @param {string} apiSecret
         * @param {string} from
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateVonageProvider(providerId, name, enabled, apiKey, apiSecret, from) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                const apiPath = '/messaging/providers/vonage/{providerId}'.replace('{providerId}', providerId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof apiKey !== 'undefined') {
                    payload['apiKey'] = apiKey;
                }
                if (typeof apiSecret !== 'undefined') {
                    payload['apiSecret'] = apiSecret;
                }
                if (typeof from !== 'undefined') {
                    payload['from'] = from;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get provider
         *
         *
         * @param {string} providerId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getProvider(providerId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                const apiPath = '/messaging/providers/{providerId}'.replace('{providerId}', providerId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete provider
         *
         *
         * @param {string} providerId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteProvider(providerId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                const apiPath = '/messaging/providers/{providerId}'.replace('{providerId}', providerId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List provider logs
         *
         *
         * @param {string} providerId
         * @param {string[]} queries
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listProviderLogs(providerId, queries) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof providerId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerId"');
                }
                const apiPath = '/messaging/providers/{providerId}/logs'.replace('{providerId}', providerId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List subscriber logs
         *
         *
         * @param {string} subscriberId
         * @param {string[]} queries
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listSubscriberLogs(subscriberId, queries) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof subscriberId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "subscriberId"');
                }
                const apiPath = '/messaging/subscribers/{subscriberId}/logs'.replace('{subscriberId}', subscriberId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List topics.
         *
         *
         * @param {string} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listTopics(queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/messaging/topics';
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create a topic.
         *
         *
         * @param {string} topicId
         * @param {string} name
         * @param {string} description
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createTopic(topicId, name, description) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof topicId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "topicId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/messaging/topics';
                const payload = {};
                if (typeof topicId !== 'undefined') {
                    payload['topicId'] = topicId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof description !== 'undefined') {
                    payload['description'] = description;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get a topic.
         *
         *
         * @param {string} topicId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getTopic(topicId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof topicId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "topicId"');
                }
                const apiPath = '/messaging/topics/{topicId}'.replace('{topicId}', topicId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update a topic.
         *
         *
         * @param {string} topicId
         * @param {string} name
         * @param {string} description
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateTopic(topicId, name, description) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof topicId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "topicId"');
                }
                const apiPath = '/messaging/topics/{topicId}'.replace('{topicId}', topicId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof description !== 'undefined') {
                    payload['description'] = description;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete a topic.
         *
         *
         * @param {string} topicId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteTopic(topicId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof topicId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "topicId"');
                }
                const apiPath = '/messaging/topics/{topicId}'.replace('{topicId}', topicId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List topic logs
         *
         *
         * @param {string} topicId
         * @param {string[]} queries
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listTopicLogs(topicId, queries) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof topicId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "topicId"');
                }
                const apiPath = '/messaging/topics/{topicId}/logs'.replace('{topicId}', topicId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List subscribers.
         *
         *
         * @param {string} topicId
         * @param {string} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listSubscribers(topicId, queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof topicId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "topicId"');
                }
                const apiPath = '/messaging/topics/{topicId}/subscribers'.replace('{topicId}', topicId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create a subscriber.
         *
         *
         * @param {string} topicId
         * @param {string} subscriberId
         * @param {string} targetId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createSubscriber(topicId, subscriberId, targetId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof topicId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "topicId"');
                }
                if (typeof subscriberId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "subscriberId"');
                }
                if (typeof targetId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "targetId"');
                }
                const apiPath = '/messaging/topics/{topicId}/subscribers'.replace('{topicId}', topicId);
                const payload = {};
                if (typeof subscriberId !== 'undefined') {
                    payload['subscriberId'] = subscriberId;
                }
                if (typeof targetId !== 'undefined') {
                    payload['targetId'] = targetId;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get a subscriber.
         *
         *
         * @param {string} topicId
         * @param {string} subscriberId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getSubscriber(topicId, subscriberId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof topicId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "topicId"');
                }
                if (typeof subscriberId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "subscriberId"');
                }
                const apiPath = '/messaging/topics/{topicId}/subscribers/{subscriberId}'.replace('{topicId}', topicId).replace('{subscriberId}', subscriberId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete a subscriber.
         *
         *
         * @param {string} topicId
         * @param {string} subscriberId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteSubscriber(topicId, subscriberId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof topicId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "topicId"');
                }
                if (typeof subscriberId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "subscriberId"');
                }
                const apiPath = '/messaging/topics/{topicId}/subscribers/{subscriberId}'.replace('{topicId}', topicId).replace('{subscriberId}', subscriberId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
    }

    class Storage extends Service {
        constructor(client) {
            super(client);
        }
        /**
         * List buckets
         *
         * Get a list of all the storage buckets. You can use the query params to
         * filter your results.
         *
         * @param {string[]} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listBuckets(queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/storage/buckets';
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create bucket
         *
         * Create a new storage bucket.
         *
         * @param {string} bucketId
         * @param {string} name
         * @param {string[]} permissions
         * @param {boolean} fileSecurity
         * @param {boolean} enabled
         * @param {number} maximumFileSize
         * @param {string[]} allowedFileExtensions
         * @param {Compression} compression
         * @param {boolean} encryption
         * @param {boolean} antivirus
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createBucket(bucketId, name, permissions, fileSecurity, enabled, maximumFileSize, allowedFileExtensions, compression, encryption, antivirus) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof bucketId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "bucketId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/storage/buckets';
                const payload = {};
                if (typeof bucketId !== 'undefined') {
                    payload['bucketId'] = bucketId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof permissions !== 'undefined') {
                    payload['permissions'] = permissions;
                }
                if (typeof fileSecurity !== 'undefined') {
                    payload['fileSecurity'] = fileSecurity;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof maximumFileSize !== 'undefined') {
                    payload['maximumFileSize'] = maximumFileSize;
                }
                if (typeof allowedFileExtensions !== 'undefined') {
                    payload['allowedFileExtensions'] = allowedFileExtensions;
                }
                if (typeof compression !== 'undefined') {
                    payload['compression'] = compression;
                }
                if (typeof encryption !== 'undefined') {
                    payload['encryption'] = encryption;
                }
                if (typeof antivirus !== 'undefined') {
                    payload['antivirus'] = antivirus;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get bucket
         *
         * Get a storage bucket by its unique ID. This endpoint response returns a
         * JSON object with the storage bucket metadata.
         *
         * @param {string} bucketId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getBucket(bucketId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof bucketId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "bucketId"');
                }
                const apiPath = '/storage/buckets/{bucketId}'.replace('{bucketId}', bucketId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update bucket
         *
         * Update a storage bucket by its unique ID.
         *
         * @param {string} bucketId
         * @param {string} name
         * @param {string[]} permissions
         * @param {boolean} fileSecurity
         * @param {boolean} enabled
         * @param {number} maximumFileSize
         * @param {string[]} allowedFileExtensions
         * @param {Compression} compression
         * @param {boolean} encryption
         * @param {boolean} antivirus
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateBucket(bucketId, name, permissions, fileSecurity, enabled, maximumFileSize, allowedFileExtensions, compression, encryption, antivirus) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof bucketId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "bucketId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/storage/buckets/{bucketId}'.replace('{bucketId}', bucketId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof permissions !== 'undefined') {
                    payload['permissions'] = permissions;
                }
                if (typeof fileSecurity !== 'undefined') {
                    payload['fileSecurity'] = fileSecurity;
                }
                if (typeof enabled !== 'undefined') {
                    payload['enabled'] = enabled;
                }
                if (typeof maximumFileSize !== 'undefined') {
                    payload['maximumFileSize'] = maximumFileSize;
                }
                if (typeof allowedFileExtensions !== 'undefined') {
                    payload['allowedFileExtensions'] = allowedFileExtensions;
                }
                if (typeof compression !== 'undefined') {
                    payload['compression'] = compression;
                }
                if (typeof encryption !== 'undefined') {
                    payload['encryption'] = encryption;
                }
                if (typeof antivirus !== 'undefined') {
                    payload['antivirus'] = antivirus;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete bucket
         *
         * Delete a storage bucket by its unique ID.
         *
         * @param {string} bucketId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteBucket(bucketId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof bucketId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "bucketId"');
                }
                const apiPath = '/storage/buckets/{bucketId}'.replace('{bucketId}', bucketId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List files
         *
         * Get a list of all the user files. You can use the query params to filter
         * your results.
         *
         * @param {string} bucketId
         * @param {string[]} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listFiles(bucketId, queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof bucketId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "bucketId"');
                }
                const apiPath = '/storage/buckets/{bucketId}/files'.replace('{bucketId}', bucketId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create file
         *
         * Create a new file. Before using this route, you should create a new bucket
         * resource using either a [server
         * integration](https://appwrite.io/docs/server/storage#storageCreateBucket)
         * API or directly from your Appwrite console.
         *
         * Larger files should be uploaded using multiple requests with the
         * [content-range](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Range)
         * header to send a partial request with a maximum supported chunk of `5MB`.
         * The `content-range` header values should always be in bytes.
         *
         * When the first request is sent, the server will return the **File** object,
         * and the subsequent part request must include the file's **id** in
         * `x-appwrite-id` header to allow the server to know that the partial upload
         * is for the existing file and not for a new one.
         *
         * If you're creating a new file using one of the Appwrite SDKs, all the
         * chunking logic will be managed by the SDK internally.
         *
         *
         * @param {string} bucketId
         * @param {string} fileId
         * @param {File} file
         * @param {string[]} permissions
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createFile(bucketId, fileId, file, permissions, onProgress = (progress) => { }) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof bucketId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "bucketId"');
                }
                if (typeof fileId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "fileId"');
                }
                if (typeof file === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "file"');
                }
                const apiPath = '/storage/buckets/{bucketId}/files'.replace('{bucketId}', bucketId);
                const payload = {};
                if (typeof fileId !== 'undefined') {
                    payload['fileId'] = fileId;
                }
                if (typeof file !== 'undefined') {
                    payload['file'] = file;
                }
                if (typeof permissions !== 'undefined') {
                    payload['permissions'] = permissions;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                if (!(file instanceof File)) {
                    throw new AppwriteException('Parameter "file" has to be a File.');
                }
                const size = file.size;
                if (size <= Service.CHUNK_SIZE) {
                    return yield this.client.call('post', uri, {
                        'content-type': 'multipart/form-data',
                    }, payload);
                }
                const apiHeaders = {
                    'content-type': 'multipart/form-data',
                };
                let offset = 0;
                let response = undefined;
                if (fileId != 'unique()') {
                    try {
                        response = yield this.client.call('GET', new URL(this.client.config.endpoint + apiPath + '/' + fileId), apiHeaders);
                        offset = response.chunksUploaded * Service.CHUNK_SIZE;
                    }
                    catch (e) {
                    }
                }
                while (offset < size) {
                    let end = Math.min(offset + Service.CHUNK_SIZE - 1, size - 1);
                    apiHeaders['content-range'] = 'bytes ' + offset + '-' + end + '/' + size;
                    if (response && response.$id) {
                        apiHeaders['x-appwrite-id'] = response.$id;
                    }
                    const chunk = file.slice(offset, end + 1);
                    payload['file'] = new File([chunk], file.name);
                    response = yield this.client.call('post', uri, apiHeaders, payload);
                    if (onProgress) {
                        onProgress({
                            $id: response.$id,
                            progress: (offset / size) * 100,
                            sizeUploaded: offset,
                            chunksTotal: response.chunksTotal,
                            chunksUploaded: response.chunksUploaded
                        });
                    }
                    offset += Service.CHUNK_SIZE;
                }
                return response;
            });
        }
        /**
         * Get file
         *
         * Get a file by its unique ID. This endpoint response returns a JSON object
         * with the file metadata.
         *
         * @param {string} bucketId
         * @param {string} fileId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getFile(bucketId, fileId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof bucketId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "bucketId"');
                }
                if (typeof fileId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "fileId"');
                }
                const apiPath = '/storage/buckets/{bucketId}/files/{fileId}'.replace('{bucketId}', bucketId).replace('{fileId}', fileId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update file
         *
         * Update a file by its unique ID. Only users with write permissions have
         * access to update this resource.
         *
         * @param {string} bucketId
         * @param {string} fileId
         * @param {string} name
         * @param {string[]} permissions
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateFile(bucketId, fileId, name, permissions) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof bucketId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "bucketId"');
                }
                if (typeof fileId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "fileId"');
                }
                const apiPath = '/storage/buckets/{bucketId}/files/{fileId}'.replace('{bucketId}', bucketId).replace('{fileId}', fileId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof permissions !== 'undefined') {
                    payload['permissions'] = permissions;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete File
         *
         * Delete a file by its unique ID. Only users with write permissions have
         * access to delete this resource.
         *
         * @param {string} bucketId
         * @param {string} fileId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteFile(bucketId, fileId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof bucketId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "bucketId"');
                }
                if (typeof fileId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "fileId"');
                }
                const apiPath = '/storage/buckets/{bucketId}/files/{fileId}'.replace('{bucketId}', bucketId).replace('{fileId}', fileId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get file for download
         *
         * Get a file content by its unique ID. The endpoint response return with a
         * 'Content-Disposition: attachment' header that tells the browser to start
         * downloading the file to user downloads directory.
         *
         * @param {string} bucketId
         * @param {string} fileId
         * @throws {AppwriteException}
         * @returns {URL}
        */
        getFileDownload(bucketId, fileId) {
            if (typeof bucketId === 'undefined') {
                throw new AppwriteException('Missing required parameter: "bucketId"');
            }
            if (typeof fileId === 'undefined') {
                throw new AppwriteException('Missing required parameter: "fileId"');
            }
            const apiPath = '/storage/buckets/{bucketId}/files/{fileId}/download'.replace('{bucketId}', bucketId).replace('{fileId}', fileId);
            const payload = {};
            const uri = new URL(this.client.config.endpoint + apiPath);
            payload['project'] = this.client.config.project;
            payload['session'] = this.client.config.session;
            for (const [key, value] of Object.entries(Service.flatten(payload))) {
                uri.searchParams.append(key, value);
            }
            return uri;
        }
        /**
         * Get file preview
         *
         * Get a file preview image. Currently, this method supports preview for image
         * files (jpg, png, and gif), other supported formats, like pdf, docs, slides,
         * and spreadsheets, will return the file icon image. You can also pass query
         * string arguments for cutting and resizing your preview image. Preview is
         * supported only for image files smaller than 10MB.
         *
         * @param {string} bucketId
         * @param {string} fileId
         * @param {number} width
         * @param {number} height
         * @param {ImageGravity} gravity
         * @param {number} quality
         * @param {number} borderWidth
         * @param {string} borderColor
         * @param {number} borderRadius
         * @param {number} opacity
         * @param {number} rotation
         * @param {string} background
         * @param {ImageFormat} output
         * @throws {AppwriteException}
         * @returns {URL}
        */
        getFilePreview(bucketId, fileId, width, height, gravity, quality, borderWidth, borderColor, borderRadius, opacity, rotation, background, output) {
            if (typeof bucketId === 'undefined') {
                throw new AppwriteException('Missing required parameter: "bucketId"');
            }
            if (typeof fileId === 'undefined') {
                throw new AppwriteException('Missing required parameter: "fileId"');
            }
            const apiPath = '/storage/buckets/{bucketId}/files/{fileId}/preview'.replace('{bucketId}', bucketId).replace('{fileId}', fileId);
            const payload = {};
            if (typeof width !== 'undefined') {
                payload['width'] = width;
            }
            if (typeof height !== 'undefined') {
                payload['height'] = height;
            }
            if (typeof gravity !== 'undefined') {
                payload['gravity'] = gravity;
            }
            if (typeof quality !== 'undefined') {
                payload['quality'] = quality;
            }
            if (typeof borderWidth !== 'undefined') {
                payload['borderWidth'] = borderWidth;
            }
            if (typeof borderColor !== 'undefined') {
                payload['borderColor'] = borderColor;
            }
            if (typeof borderRadius !== 'undefined') {
                payload['borderRadius'] = borderRadius;
            }
            if (typeof opacity !== 'undefined') {
                payload['opacity'] = opacity;
            }
            if (typeof rotation !== 'undefined') {
                payload['rotation'] = rotation;
            }
            if (typeof background !== 'undefined') {
                payload['background'] = background;
            }
            if (typeof output !== 'undefined') {
                payload['output'] = output;
            }
            const uri = new URL(this.client.config.endpoint + apiPath);
            payload['project'] = this.client.config.project;
            payload['session'] = this.client.config.session;
            for (const [key, value] of Object.entries(Service.flatten(payload))) {
                uri.searchParams.append(key, value);
            }
            return uri;
        }
        /**
         * Get file for view
         *
         * Get a file content by its unique ID. This endpoint is similar to the
         * download method but returns with no  'Content-Disposition: attachment'
         * header.
         *
         * @param {string} bucketId
         * @param {string} fileId
         * @throws {AppwriteException}
         * @returns {URL}
        */
        getFileView(bucketId, fileId) {
            if (typeof bucketId === 'undefined') {
                throw new AppwriteException('Missing required parameter: "bucketId"');
            }
            if (typeof fileId === 'undefined') {
                throw new AppwriteException('Missing required parameter: "fileId"');
            }
            const apiPath = '/storage/buckets/{bucketId}/files/{fileId}/view'.replace('{bucketId}', bucketId).replace('{fileId}', fileId);
            const payload = {};
            const uri = new URL(this.client.config.endpoint + apiPath);
            payload['project'] = this.client.config.project;
            payload['session'] = this.client.config.session;
            for (const [key, value] of Object.entries(Service.flatten(payload))) {
                uri.searchParams.append(key, value);
            }
            return uri;
        }
    }

    class Teams extends Service {
        constructor(client) {
            super(client);
        }
        /**
         * List teams
         *
         * Get a list of all the teams in which the current user is a member. You can
         * use the parameters to filter your results.
         *
         * @param {string[]} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        list(queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/teams';
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create team
         *
         * Create a new team. The user who creates the team will automatically be
         * assigned as the owner of the team. Only the users with the owner role can
         * invite new members, add new owners and delete or update the team.
         *
         * @param {string} teamId
         * @param {string} name
         * @param {string[]} roles
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        create(teamId, name, roles) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof teamId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "teamId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/teams';
                const payload = {};
                if (typeof teamId !== 'undefined') {
                    payload['teamId'] = teamId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                if (typeof roles !== 'undefined') {
                    payload['roles'] = roles;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get team
         *
         * Get a team by its ID. All team members have read access for this resource.
         *
         * @param {string} teamId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        get(teamId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof teamId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "teamId"');
                }
                const apiPath = '/teams/{teamId}'.replace('{teamId}', teamId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update name
         *
         * Update the team's name by its unique ID.
         *
         * @param {string} teamId
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateName(teamId, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof teamId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "teamId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/teams/{teamId}'.replace('{teamId}', teamId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete team
         *
         * Delete a team using its ID. Only team members with the owner role can
         * delete the team.
         *
         * @param {string} teamId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        delete(teamId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof teamId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "teamId"');
                }
                const apiPath = '/teams/{teamId}'.replace('{teamId}', teamId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List team memberships
         *
         * Use this endpoint to list a team's members using the team's ID. All team
         * members have read access to this endpoint.
         *
         * @param {string} teamId
         * @param {string[]} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listMemberships(teamId, queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof teamId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "teamId"');
                }
                const apiPath = '/teams/{teamId}/memberships'.replace('{teamId}', teamId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create team membership
         *
         * Invite a new member to join your team. Provide an ID for existing users, or
         * invite unregistered users using an email or phone number. If initiated from
         * a Client SDK, Appwrite will send an email or sms with a link to join the
         * team to the invited user, and an account will be created for them if one
         * doesn't exist. If initiated from a Server SDK, the new member will be added
         * automatically to the team.
         *
         * You only need to provide one of a user ID, email, or phone number. Appwrite
         * will prioritize accepting the user ID > email > phone number if you provide
         * more than one of these parameters.
         *
         * Use the `url` parameter to redirect the user from the invitation email to
         * your app. After the user is redirected, use the [Update Team Membership
         * Status](https://appwrite.io/docs/references/cloud/client-web/teams#updateMembershipStatus)
         * endpoint to allow the user to accept the invitation to the team.
         *
         * Please note that to avoid a [Redirect
         * Attack](https://github.com/OWASP/CheatSheetSeries/blob/master/cheatsheets/Unvalidated_Redirects_and_Forwards_Cheat_Sheet.md)
         * Appwrite will accept the only redirect URLs under the domains you have
         * added as a platform on the Appwrite Console.
         *
         *
         * @param {string} teamId
         * @param {string[]} roles
         * @param {string} email
         * @param {string} userId
         * @param {string} phone
         * @param {string} url
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createMembership(teamId, roles, email, userId, phone, url, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof teamId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "teamId"');
                }
                if (typeof roles === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "roles"');
                }
                const apiPath = '/teams/{teamId}/memberships'.replace('{teamId}', teamId);
                const payload = {};
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof phone !== 'undefined') {
                    payload['phone'] = phone;
                }
                if (typeof roles !== 'undefined') {
                    payload['roles'] = roles;
                }
                if (typeof url !== 'undefined') {
                    payload['url'] = url;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get team membership
         *
         * Get a team member by the membership unique id. All team members have read
         * access for this resource.
         *
         * @param {string} teamId
         * @param {string} membershipId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getMembership(teamId, membershipId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof teamId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "teamId"');
                }
                if (typeof membershipId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "membershipId"');
                }
                const apiPath = '/teams/{teamId}/memberships/{membershipId}'.replace('{teamId}', teamId).replace('{membershipId}', membershipId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update membership
         *
         * Modify the roles of a team member. Only team members with the owner role
         * have access to this endpoint. Learn more about [roles and
         * permissions](https://appwrite.io/docs/permissions).
         *
         *
         * @param {string} teamId
         * @param {string} membershipId
         * @param {string[]} roles
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateMembership(teamId, membershipId, roles) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof teamId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "teamId"');
                }
                if (typeof membershipId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "membershipId"');
                }
                if (typeof roles === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "roles"');
                }
                const apiPath = '/teams/{teamId}/memberships/{membershipId}'.replace('{teamId}', teamId).replace('{membershipId}', membershipId);
                const payload = {};
                if (typeof roles !== 'undefined') {
                    payload['roles'] = roles;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete team membership
         *
         * This endpoint allows a user to leave a team or for a team owner to delete
         * the membership of any other team member. You can also use this endpoint to
         * delete a user membership even if it is not accepted.
         *
         * @param {string} teamId
         * @param {string} membershipId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteMembership(teamId, membershipId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof teamId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "teamId"');
                }
                if (typeof membershipId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "membershipId"');
                }
                const apiPath = '/teams/{teamId}/memberships/{membershipId}'.replace('{teamId}', teamId).replace('{membershipId}', membershipId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update team membership status
         *
         * Use this endpoint to allow a user to accept an invitation to join a team
         * after being redirected back to your app from the invitation email received
         * by the user.
         *
         * If the request is successful, a session for the user is automatically
         * created.
         *
         *
         * @param {string} teamId
         * @param {string} membershipId
         * @param {string} userId
         * @param {string} secret
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateMembershipStatus(teamId, membershipId, userId, secret) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof teamId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "teamId"');
                }
                if (typeof membershipId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "membershipId"');
                }
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof secret === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "secret"');
                }
                const apiPath = '/teams/{teamId}/memberships/{membershipId}/status'.replace('{teamId}', teamId).replace('{membershipId}', membershipId);
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof secret !== 'undefined') {
                    payload['secret'] = secret;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get team preferences
         *
         * Get the team's shared preferences by its unique ID. If a preference doesn't
         * need to be shared by all team members, prefer storing them in [user
         * preferences](https://appwrite.io/docs/references/cloud/client-web/account#getPrefs).
         *
         * @param {string} teamId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getPrefs(teamId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof teamId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "teamId"');
                }
                const apiPath = '/teams/{teamId}/prefs'.replace('{teamId}', teamId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update preferences
         *
         * Update the team's preferences by its unique ID. The object you pass is
         * stored as is and replaces any previous value. The maximum allowed prefs
         * size is 64kB and throws an error if exceeded.
         *
         * @param {string} teamId
         * @param {object} prefs
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updatePrefs(teamId, prefs) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof teamId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "teamId"');
                }
                if (typeof prefs === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "prefs"');
                }
                const apiPath = '/teams/{teamId}/prefs'.replace('{teamId}', teamId);
                const payload = {};
                if (typeof prefs !== 'undefined') {
                    payload['prefs'] = prefs;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
    }

    class Users extends Service {
        constructor(client) {
            super(client);
        }
        /**
         * List users
         *
         * Get a list of all the project's users. You can use the query params to
         * filter your results.
         *
         * @param {string[]} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        list(queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/users';
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create user
         *
         * Create a new user.
         *
         * @param {string} userId
         * @param {string} email
         * @param {string} phone
         * @param {string} password
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        create(userId, email, phone, password, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                const apiPath = '/users';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof phone !== 'undefined') {
                    payload['phone'] = phone;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create user with Argon2 password
         *
         * Create a new user. Password provided must be hashed with the
         * [Argon2](https://en.wikipedia.org/wiki/Argon2) algorithm. Use the [POST
         * /users](https://appwrite.io/docs/server/users#usersCreate) endpoint to
         * create users with a plain text password.
         *
         * @param {string} userId
         * @param {string} email
         * @param {string} password
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createArgon2User(userId, email, password, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                const apiPath = '/users/argon2';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create user with bcrypt password
         *
         * Create a new user. Password provided must be hashed with the
         * [Bcrypt](https://en.wikipedia.org/wiki/Bcrypt) algorithm. Use the [POST
         * /users](https://appwrite.io/docs/server/users#usersCreate) endpoint to
         * create users with a plain text password.
         *
         * @param {string} userId
         * @param {string} email
         * @param {string} password
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createBcryptUser(userId, email, password, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                const apiPath = '/users/bcrypt';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List Identities
         *
         * Get identities for all users.
         *
         * @param {string} queries
         * @param {string} search
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listIdentities(queries, search) {
            return __awaiter(this, void 0, void 0, function* () {
                const apiPath = '/users/identities';
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                if (typeof search !== 'undefined') {
                    payload['search'] = search;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete identity
         *
         * Delete an identity by its unique ID.
         *
         * @param {string} identityId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteIdentity(identityId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof identityId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "identityId"');
                }
                const apiPath = '/users/identities/{identityId}'.replace('{identityId}', identityId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create user with MD5 password
         *
         * Create a new user. Password provided must be hashed with the
         * [MD5](https://en.wikipedia.org/wiki/MD5) algorithm. Use the [POST
         * /users](https://appwrite.io/docs/server/users#usersCreate) endpoint to
         * create users with a plain text password.
         *
         * @param {string} userId
         * @param {string} email
         * @param {string} password
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createMD5User(userId, email, password, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                const apiPath = '/users/md5';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create user with PHPass password
         *
         * Create a new user. Password provided must be hashed with the
         * [PHPass](https://www.openwall.com/phpass/) algorithm. Use the [POST
         * /users](https://appwrite.io/docs/server/users#usersCreate) endpoint to
         * create users with a plain text password.
         *
         * @param {string} userId
         * @param {string} email
         * @param {string} password
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createPHPassUser(userId, email, password, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                const apiPath = '/users/phpass';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create user with Scrypt password
         *
         * Create a new user. Password provided must be hashed with the
         * [Scrypt](https://github.com/Tarsnap/scrypt) algorithm. Use the [POST
         * /users](https://appwrite.io/docs/server/users#usersCreate) endpoint to
         * create users with a plain text password.
         *
         * @param {string} userId
         * @param {string} email
         * @param {string} password
         * @param {string} passwordSalt
         * @param {number} passwordCpu
         * @param {number} passwordMemory
         * @param {number} passwordParallel
         * @param {number} passwordLength
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createScryptUser(userId, email, password, passwordSalt, passwordCpu, passwordMemory, passwordParallel, passwordLength, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                if (typeof passwordSalt === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "passwordSalt"');
                }
                if (typeof passwordCpu === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "passwordCpu"');
                }
                if (typeof passwordMemory === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "passwordMemory"');
                }
                if (typeof passwordParallel === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "passwordParallel"');
                }
                if (typeof passwordLength === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "passwordLength"');
                }
                const apiPath = '/users/scrypt';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof passwordSalt !== 'undefined') {
                    payload['passwordSalt'] = passwordSalt;
                }
                if (typeof passwordCpu !== 'undefined') {
                    payload['passwordCpu'] = passwordCpu;
                }
                if (typeof passwordMemory !== 'undefined') {
                    payload['passwordMemory'] = passwordMemory;
                }
                if (typeof passwordParallel !== 'undefined') {
                    payload['passwordParallel'] = passwordParallel;
                }
                if (typeof passwordLength !== 'undefined') {
                    payload['passwordLength'] = passwordLength;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create user with Scrypt modified password
         *
         * Create a new user. Password provided must be hashed with the [Scrypt
         * Modified](https://gist.github.com/Meldiron/eecf84a0225eccb5a378d45bb27462cc)
         * algorithm. Use the [POST
         * /users](https://appwrite.io/docs/server/users#usersCreate) endpoint to
         * create users with a plain text password.
         *
         * @param {string} userId
         * @param {string} email
         * @param {string} password
         * @param {string} passwordSalt
         * @param {string} passwordSaltSeparator
         * @param {string} passwordSignerKey
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createScryptModifiedUser(userId, email, password, passwordSalt, passwordSaltSeparator, passwordSignerKey, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                if (typeof passwordSalt === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "passwordSalt"');
                }
                if (typeof passwordSaltSeparator === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "passwordSaltSeparator"');
                }
                if (typeof passwordSignerKey === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "passwordSignerKey"');
                }
                const apiPath = '/users/scrypt-modified';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof passwordSalt !== 'undefined') {
                    payload['passwordSalt'] = passwordSalt;
                }
                if (typeof passwordSaltSeparator !== 'undefined') {
                    payload['passwordSaltSeparator'] = passwordSaltSeparator;
                }
                if (typeof passwordSignerKey !== 'undefined') {
                    payload['passwordSignerKey'] = passwordSignerKey;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create user with SHA password
         *
         * Create a new user. Password provided must be hashed with the
         * [SHA](https://en.wikipedia.org/wiki/Secure_Hash_Algorithm) algorithm. Use
         * the [POST /users](https://appwrite.io/docs/server/users#usersCreate)
         * endpoint to create users with a plain text password.
         *
         * @param {string} userId
         * @param {string} email
         * @param {string} password
         * @param {PasswordVersion} passwordVersion
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createSHAUser(userId, email, password, passwordVersion, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                const apiPath = '/users/sha';
                const payload = {};
                if (typeof userId !== 'undefined') {
                    payload['userId'] = userId;
                }
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                if (typeof passwordVersion !== 'undefined') {
                    payload['passwordVersion'] = passwordVersion;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get user
         *
         * Get a user by its unique ID.
         *
         * @param {string} userId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        get(userId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                const apiPath = '/users/{userId}'.replace('{userId}', userId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete user
         *
         * Delete a user by its unique ID, thereby releasing it's ID. Since ID is
         * released and can be reused, all user-related resources like documents or
         * storage files should be deleted before user deletion. If you want to keep
         * ID reserved, use the
         * [updateStatus](https://appwrite.io/docs/server/users#usersUpdateStatus)
         * endpoint instead.
         *
         * @param {string} userId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        delete(userId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                const apiPath = '/users/{userId}'.replace('{userId}', userId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update email
         *
         * Update the user email by its unique ID.
         *
         * @param {string} userId
         * @param {string} email
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateEmail(userId, email) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof email === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "email"');
                }
                const apiPath = '/users/{userId}/email'.replace('{userId}', userId);
                const payload = {};
                if (typeof email !== 'undefined') {
                    payload['email'] = email;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update user labels
         *
         * Update the user labels by its unique ID.
         *
         * Labels can be used to grant access to resources. While teams are a way for
         * user's to share access to a resource, labels can be defined by the
         * developer to grant access without an invitation. See the [Permissions
         * docs](https://appwrite.io/docs/permissions) for more info.
         *
         * @param {string} userId
         * @param {string[]} labels
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateLabels(userId, labels) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof labels === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "labels"');
                }
                const apiPath = '/users/{userId}/labels'.replace('{userId}', userId);
                const payload = {};
                if (typeof labels !== 'undefined') {
                    payload['labels'] = labels;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('put', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List user logs
         *
         * Get the user activity logs list by its unique ID.
         *
         * @param {string} userId
         * @param {string[]} queries
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listLogs(userId, queries) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                const apiPath = '/users/{userId}/logs'.replace('{userId}', userId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List user memberships
         *
         * Get the user membership list by its unique ID.
         *
         * @param {string} userId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listMemberships(userId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                const apiPath = '/users/{userId}/memberships'.replace('{userId}', userId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update name
         *
         * Update the user name by its unique ID.
         *
         * @param {string} userId
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateName(userId, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof name === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "name"');
                }
                const apiPath = '/users/{userId}/name'.replace('{userId}', userId);
                const payload = {};
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update password
         *
         * Update the user password by its unique ID.
         *
         * @param {string} userId
         * @param {string} password
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updatePassword(userId, password) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof password === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "password"');
                }
                const apiPath = '/users/{userId}/password'.replace('{userId}', userId);
                const payload = {};
                if (typeof password !== 'undefined') {
                    payload['password'] = password;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update phone
         *
         * Update the user phone by its unique ID.
         *
         * @param {string} userId
         * @param {string} number
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updatePhone(userId, number) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof number === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "number"');
                }
                const apiPath = '/users/{userId}/phone'.replace('{userId}', userId);
                const payload = {};
                if (typeof number !== 'undefined') {
                    payload['number'] = number;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get user preferences
         *
         * Get the user preferences by its unique ID.
         *
         * @param {string} userId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getPrefs(userId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                const apiPath = '/users/{userId}/prefs'.replace('{userId}', userId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update user preferences
         *
         * Update the user preferences by its unique ID. The object you pass is stored
         * as is, and replaces any previous value. The maximum allowed prefs size is
         * 64kB and throws error if exceeded.
         *
         * @param {string} userId
         * @param {object} prefs
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updatePrefs(userId, prefs) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof prefs === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "prefs"');
                }
                const apiPath = '/users/{userId}/prefs'.replace('{userId}', userId);
                const payload = {};
                if (typeof prefs !== 'undefined') {
                    payload['prefs'] = prefs;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List user sessions
         *
         * Get the user sessions list by its unique ID.
         *
         * @param {string} userId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listSessions(userId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                const apiPath = '/users/{userId}/sessions'.replace('{userId}', userId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create session
         *
         *
         * @param {string} userId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createSession(userId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                const apiPath = '/users/{userId}/sessions'.replace('{userId}', userId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete user sessions
         *
         * Delete all user's sessions by using the user's unique ID.
         *
         * @param {string} userId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteSessions(userId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                const apiPath = '/users/{userId}/sessions'.replace('{userId}', userId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete user session
         *
         * Delete a user sessions by its unique ID.
         *
         * @param {string} userId
         * @param {string} sessionId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteSession(userId, sessionId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof sessionId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "sessionId"');
                }
                const apiPath = '/users/{userId}/sessions/{sessionId}'.replace('{userId}', userId).replace('{sessionId}', sessionId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update user status
         *
         * Update the user status by its unique ID. Use this endpoint as an
         * alternative to deleting a user if you want to keep user's ID reserved.
         *
         * @param {string} userId
         * @param {boolean} status
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateStatus(userId, status) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof status === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "status"');
                }
                const apiPath = '/users/{userId}/status'.replace('{userId}', userId);
                const payload = {};
                if (typeof status !== 'undefined') {
                    payload['status'] = status;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * List User Targets
         *
         *
         * @param {string} userId
         * @param {string} queries
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        listTargets(userId, queries) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                const apiPath = '/users/{userId}/targets'.replace('{userId}', userId);
                const payload = {};
                if (typeof queries !== 'undefined') {
                    payload['queries'] = queries;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create User Target
         *
         *
         * @param {string} userId
         * @param {string} targetId
         * @param {ProviderType} providerType
         * @param {string} identifier
         * @param {string} providerId
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createTarget(userId, targetId, providerType, identifier, providerId, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof targetId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "targetId"');
                }
                if (typeof providerType === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "providerType"');
                }
                if (typeof identifier === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "identifier"');
                }
                const apiPath = '/users/{userId}/targets'.replace('{userId}', userId);
                const payload = {};
                if (typeof targetId !== 'undefined') {
                    payload['targetId'] = targetId;
                }
                if (typeof providerType !== 'undefined') {
                    payload['providerType'] = providerType;
                }
                if (typeof identifier !== 'undefined') {
                    payload['identifier'] = identifier;
                }
                if (typeof providerId !== 'undefined') {
                    payload['providerId'] = providerId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Get User Target
         *
         *
         * @param {string} userId
         * @param {string} targetId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        getTarget(userId, targetId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof targetId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "targetId"');
                }
                const apiPath = '/users/{userId}/targets/{targetId}'.replace('{userId}', userId).replace('{targetId}', targetId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('get', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update User target
         *
         *
         * @param {string} userId
         * @param {string} targetId
         * @param {string} identifier
         * @param {string} providerId
         * @param {string} name
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateTarget(userId, targetId, identifier, providerId, name) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof targetId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "targetId"');
                }
                const apiPath = '/users/{userId}/targets/{targetId}'.replace('{userId}', userId).replace('{targetId}', targetId);
                const payload = {};
                if (typeof identifier !== 'undefined') {
                    payload['identifier'] = identifier;
                }
                if (typeof providerId !== 'undefined') {
                    payload['providerId'] = providerId;
                }
                if (typeof name !== 'undefined') {
                    payload['name'] = name;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Delete user target
         *
         *
         * @param {string} userId
         * @param {string} targetId
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        deleteTarget(userId, targetId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof targetId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "targetId"');
                }
                const apiPath = '/users/{userId}/targets/{targetId}'.replace('{userId}', userId).replace('{targetId}', targetId);
                const payload = {};
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('delete', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Create token
         *
         * Returns a custom token with a secret key for creating a session. If the
         * provided user ID has not be registered, a new user will be created. Use the
         * returned user ID and secret and submit a request to the [PUT
         * /account/sessions/custom](https://appwrite.io/docs/references/cloud/client-web/account#updateCustomSession)
         * endpoint to complete the login process. By default, the secret sent in the
         * response is valid for 15 minutes.
         *
         * @param {string} userId
         * @param {number} length
         * @param {number} expire
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        createToken(userId, length, expire) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                const apiPath = '/users/{userId}/tokens'.replace('{userId}', userId);
                const payload = {};
                if (typeof length !== 'undefined') {
                    payload['length'] = length;
                }
                if (typeof expire !== 'undefined') {
                    payload['expire'] = expire;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('post', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update email verification
         *
         * Update the user email verification status by its unique ID.
         *
         * @param {string} userId
         * @param {boolean} emailVerification
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updateEmailVerification(userId, emailVerification) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof emailVerification === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "emailVerification"');
                }
                const apiPath = '/users/{userId}/verification'.replace('{userId}', userId);
                const payload = {};
                if (typeof emailVerification !== 'undefined') {
                    payload['emailVerification'] = emailVerification;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
        /**
         * Update phone verification
         *
         * Update the user phone verification status by its unique ID.
         *
         * @param {string} userId
         * @param {boolean} phoneVerification
         * @throws {AppwriteException}
         * @returns {Promise}
        */
        updatePhoneVerification(userId, phoneVerification) {
            return __awaiter(this, void 0, void 0, function* () {
                if (typeof userId === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "userId"');
                }
                if (typeof phoneVerification === 'undefined') {
                    throw new AppwriteException('Missing required parameter: "phoneVerification"');
                }
                const apiPath = '/users/{userId}/verification/phone'.replace('{userId}', userId);
                const payload = {};
                if (typeof phoneVerification !== 'undefined') {
                    payload['phoneVerification'] = phoneVerification;
                }
                const uri = new URL(this.client.config.endpoint + apiPath);
                return yield this.client.call('patch', uri, {
                    'content-type': 'application/json',
                }, payload);
            });
        }
    }

    class Permission {
    }
    Permission.read = (role) => {
        return `read("${role}")`;
    };
    Permission.write = (role) => {
        return `write("${role}")`;
    };
    Permission.create = (role) => {
        return `create("${role}")`;
    };
    Permission.update = (role) => {
        return `update("${role}")`;
    };
    Permission.delete = (role) => {
        return `delete("${role}")`;
    };

    /**
     * Helper class to generate role strings for `Permission`.
     */
    class Role {
        /**
         * Grants access to anyone.
         *
         * This includes authenticated and unauthenticated users.
         *
         * @returns {string}
         */
        static any() {
            return 'any';
        }
        /**
         * Grants access to a specific user by user ID.
         *
         * You can optionally pass verified or unverified for
         * `status` to target specific types of users.
         *
         * @param {string} id
         * @param {string} status
         * @returns {string}
         */
        static user(id, status = '') {
            if (status === '') {
                return `user:${id}`;
            }
            return `user:${id}/${status}`;
        }
        /**
         * Grants access to any authenticated or anonymous user.
         *
         * You can optionally pass verified or unverified for
         * `status` to target specific types of users.
         *
         * @param {string} status
         * @returns {string}
         */
        static users(status = '') {
            if (status === '') {
                return 'users';
            }
            return `users/${status}`;
        }
        /**
         * Grants access to any guest user without a session.
         *
         * Authenticated users don't have access to this role.
         *
         * @returns {string}
         */
        static guests() {
            return 'guests';
        }
        /**
         * Grants access to a team by team ID.
         *
         * You can optionally pass a role for `role` to target
         * team members with the specified role.
         *
         * @param {string} id
         * @param {string} role
         * @returns {string}
         */
        static team(id, role = '') {
            if (role === '') {
                return `team:${id}`;
            }
            return `team:${id}/${role}`;
        }
        /**
         * Grants access to a specific member of a team.
         *
         * When the member is removed from the team, they will
         * no longer have access.
         *
         * @param {string} id
         * @returns {string}
         */
        static member(id) {
            return `member:${id}`;
        }
        /**
         * Grants access to a user with the specified label.
         *
         * @param {string} name
         * @returns  {string}
         */
        static label(name) {
            return `label:${name}`;
        }
    }

    class ID {
        static custom(id) {
            return id;
        }
        static unique() {
            return 'unique()';
        }
    }

    exports.Provider = void 0;
    (function (Provider) {
        Provider["Amazon"] = "amazon";
        Provider["Apple"] = "apple";
        Provider["Auth0"] = "auth0";
        Provider["Authentik"] = "authentik";
        Provider["Autodesk"] = "autodesk";
        Provider["Bitbucket"] = "bitbucket";
        Provider["Bitly"] = "bitly";
        Provider["Box"] = "box";
        Provider["Dailymotion"] = "dailymotion";
        Provider["Discord"] = "discord";
        Provider["Disqus"] = "disqus";
        Provider["Dropbox"] = "dropbox";
        Provider["Etsy"] = "etsy";
        Provider["Facebook"] = "facebook";
        Provider["Github"] = "github";
        Provider["Gitlab"] = "gitlab";
        Provider["Google"] = "google";
        Provider["Linkedin"] = "linkedin";
        Provider["Microsoft"] = "microsoft";
        Provider["Notion"] = "notion";
        Provider["Oidc"] = "oidc";
        Provider["Okta"] = "okta";
        Provider["Paypal"] = "paypal";
        Provider["PaypalSandbox"] = "paypalSandbox";
        Provider["Podio"] = "podio";
        Provider["Salesforce"] = "salesforce";
        Provider["Slack"] = "slack";
        Provider["Spotify"] = "spotify";
        Provider["Stripe"] = "stripe";
        Provider["Tradeshift"] = "tradeshift";
        Provider["TradeshiftBox"] = "tradeshiftBox";
        Provider["Twitch"] = "twitch";
        Provider["Wordpress"] = "wordpress";
        Provider["Yahoo"] = "yahoo";
        Provider["Yammer"] = "yammer";
        Provider["Yandex"] = "yandex";
        Provider["Zoho"] = "zoho";
        Provider["Zoom"] = "zoom";
        Provider["Mock"] = "mock";
    })(exports.Provider || (exports.Provider = {}));

    exports.Browser = void 0;
    (function (Browser) {
        Browser["AvantBrowser"] = "aa";
        Browser["AndroidWebViewBeta"] = "an";
        Browser["GoogleChrome"] = "ch";
        Browser["GoogleChromeIOS"] = "ci";
        Browser["GoogleChromeMobile"] = "cm";
        Browser["Chromium"] = "cr";
        Browser["MozillaFirefox"] = "ff";
        Browser["Safari"] = "sf";
        Browser["MobileSafari"] = "mf";
        Browser["MicrosoftEdge"] = "ps";
        Browser["MicrosoftEdgeIOS"] = "oi";
        Browser["OperaMini"] = "om";
        Browser["Opera"] = "op";
        Browser["OperaNext"] = "on";
    })(exports.Browser || (exports.Browser = {}));

    exports.CreditCard = void 0;
    (function (CreditCard) {
        CreditCard["AmericanExpress"] = "amex";
        CreditCard["Argencard"] = "argencard";
        CreditCard["Cabal"] = "cabal";
        CreditCard["Consosud"] = "censosud";
        CreditCard["DinersClub"] = "diners";
        CreditCard["Discover"] = "discover";
        CreditCard["Elo"] = "elo";
        CreditCard["Hipercard"] = "hipercard";
        CreditCard["JCB"] = "jcb";
        CreditCard["Mastercard"] = "mastercard";
        CreditCard["Naranja"] = "naranja";
        CreditCard["TarjetaShopping"] = "targeta-shopping";
        CreditCard["UnionChinaPay"] = "union-china-pay";
        CreditCard["Visa"] = "visa";
        CreditCard["MIR"] = "mir";
        CreditCard["Maestro"] = "maestro";
    })(exports.CreditCard || (exports.CreditCard = {}));

    exports.Flag = void 0;
    (function (Flag) {
        Flag["Afghanistan"] = "af";
        Flag["Angola"] = "ao";
        Flag["Albania"] = "al";
        Flag["Andorra"] = "ad";
        Flag["UnitedArabEmirates"] = "ae";
        Flag["Argentina"] = "ar";
        Flag["Armenia"] = "am";
        Flag["AntiguaAndBarbuda"] = "ag";
        Flag["Australia"] = "au";
        Flag["Austria"] = "at";
        Flag["Azerbaijan"] = "az";
        Flag["Burundi"] = "bi";
        Flag["Belgium"] = "be";
        Flag["Benin"] = "bj";
        Flag["BurkinaFaso"] = "bf";
        Flag["Bangladesh"] = "bd";
        Flag["Bulgaria"] = "bg";
        Flag["Bahrain"] = "bh";
        Flag["Bahamas"] = "bs";
        Flag["BosniaAndHerzegovina"] = "ba";
        Flag["Belarus"] = "by";
        Flag["Belize"] = "bz";
        Flag["Bolivia"] = "bo";
        Flag["Brazil"] = "br";
        Flag["Barbados"] = "bb";
        Flag["BruneiDarussalam"] = "bn";
        Flag["Bhutan"] = "bt";
        Flag["Botswana"] = "bw";
        Flag["CentralAfricanRepublic"] = "cf";
        Flag["Canada"] = "ca";
        Flag["Switzerland"] = "ch";
        Flag["Chile"] = "cl";
        Flag["China"] = "cn";
        Flag["CTeDIvoire"] = "ci";
        Flag["Cameroon"] = "cm";
        Flag["DemocraticRepublicOfTheCongo"] = "cd";
        Flag["RepublicOfTheCongo"] = "cg";
        Flag["Colombia"] = "co";
        Flag["Comoros"] = "km";
        Flag["CapeVerde"] = "cv";
        Flag["CostaRica"] = "cr";
        Flag["Cuba"] = "cu";
        Flag["Cyprus"] = "cy";
        Flag["CzechRepublic"] = "cz";
        Flag["Germany"] = "de";
        Flag["Djibouti"] = "dj";
        Flag["Dominica"] = "dm";
        Flag["Denmark"] = "dk";
        Flag["DominicanRepublic"] = "do";
        Flag["Algeria"] = "dz";
        Flag["Ecuador"] = "ec";
        Flag["Egypt"] = "eg";
        Flag["Eritrea"] = "er";
        Flag["Spain"] = "es";
        Flag["Estonia"] = "ee";
        Flag["Ethiopia"] = "et";
        Flag["Finland"] = "fi";
        Flag["Fiji"] = "fj";
        Flag["France"] = "fr";
        Flag["MicronesiaFederatedStatesOf"] = "fm";
        Flag["Gabon"] = "ga";
        Flag["UnitedKingdom"] = "gb";
        Flag["Georgia"] = "ge";
        Flag["Ghana"] = "gh";
        Flag["Guinea"] = "gn";
        Flag["Gambia"] = "gm";
        Flag["GuineaBissau"] = "gw";
        Flag["EquatorialGuinea"] = "gq";
        Flag["Greece"] = "gr";
        Flag["Grenada"] = "gd";
        Flag["Guatemala"] = "gt";
        Flag["Guyana"] = "gy";
        Flag["Honduras"] = "hn";
        Flag["Croatia"] = "hr";
        Flag["Haiti"] = "ht";
        Flag["Hungary"] = "hu";
        Flag["Indonesia"] = "id";
        Flag["India"] = "in";
        Flag["Ireland"] = "ie";
        Flag["IranIslamicRepublicOf"] = "ir";
        Flag["Iraq"] = "iq";
        Flag["Iceland"] = "is";
        Flag["Israel"] = "il";
        Flag["Italy"] = "it";
        Flag["Jamaica"] = "jm";
        Flag["Jordan"] = "jo";
        Flag["Japan"] = "jp";
        Flag["Kazakhstan"] = "kz";
        Flag["Kenya"] = "ke";
        Flag["Kyrgyzstan"] = "kg";
        Flag["Cambodia"] = "kh";
        Flag["Kiribati"] = "ki";
        Flag["SaintKittsAndNevis"] = "kn";
        Flag["SouthKorea"] = "kr";
        Flag["Kuwait"] = "kw";
        Flag["LaoPeopleSDemocraticRepublic"] = "la";
        Flag["Lebanon"] = "lb";
        Flag["Liberia"] = "lr";
        Flag["Libya"] = "ly";
        Flag["SaintLucia"] = "lc";
        Flag["Liechtenstein"] = "li";
        Flag["SriLanka"] = "lk";
        Flag["Lesotho"] = "ls";
        Flag["Lithuania"] = "lt";
        Flag["Luxembourg"] = "lu";
        Flag["Latvia"] = "lv";
        Flag["Morocco"] = "ma";
        Flag["Monaco"] = "mc";
        Flag["Moldova"] = "md";
        Flag["Madagascar"] = "mg";
        Flag["Maldives"] = "mv";
        Flag["Mexico"] = "mx";
        Flag["MarshallIslands"] = "mh";
        Flag["NorthMacedonia"] = "mk";
        Flag["Mali"] = "ml";
        Flag["Malta"] = "mt";
        Flag["Myanmar"] = "mm";
        Flag["Montenegro"] = "me";
        Flag["Mongolia"] = "mn";
        Flag["Mozambique"] = "mz";
        Flag["Mauritania"] = "mr";
        Flag["Mauritius"] = "mu";
        Flag["Malawi"] = "mw";
        Flag["Malaysia"] = "my";
        Flag["Namibia"] = "na";
        Flag["Niger"] = "ne";
        Flag["Nigeria"] = "ng";
        Flag["Nicaragua"] = "ni";
        Flag["Netherlands"] = "nl";
        Flag["Norway"] = "no";
        Flag["Nepal"] = "np";
        Flag["Nauru"] = "nr";
        Flag["NewZealand"] = "nz";
        Flag["Oman"] = "om";
        Flag["Pakistan"] = "pk";
        Flag["Panama"] = "pa";
        Flag["Peru"] = "pe";
        Flag["Philippines"] = "ph";
        Flag["Palau"] = "pw";
        Flag["PapuaNewGuinea"] = "pg";
        Flag["Poland"] = "pl";
        Flag["NorthKorea"] = "kp";
        Flag["Portugal"] = "pt";
        Flag["Paraguay"] = "py";
        Flag["Qatar"] = "qa";
        Flag["Romania"] = "ro";
        Flag["Russia"] = "ru";
        Flag["Rwanda"] = "rw";
        Flag["SaudiArabia"] = "sa";
        Flag["Sudan"] = "sd";
        Flag["Senegal"] = "sn";
        Flag["Singapore"] = "sg";
        Flag["SolomonIslands"] = "sb";
        Flag["SierraLeone"] = "sl";
        Flag["ElSalvador"] = "sv";
        Flag["SanMarino"] = "sm";
        Flag["Somalia"] = "so";
        Flag["Serbia"] = "rs";
        Flag["SouthSudan"] = "ss";
        Flag["SaoTomeAndPrincipe"] = "st";
        Flag["Suriname"] = "sr";
        Flag["Slovakia"] = "sk";
        Flag["Slovenia"] = "si";
        Flag["Sweden"] = "se";
        Flag["Eswatini"] = "sz";
        Flag["Seychelles"] = "sc";
        Flag["Syria"] = "sy";
        Flag["Chad"] = "td";
        Flag["Togo"] = "tg";
        Flag["Thailand"] = "th";
        Flag["Tajikistan"] = "tj";
        Flag["Turkmenistan"] = "tm";
        Flag["TimorLeste"] = "tl";
        Flag["Tonga"] = "to";
        Flag["TrinidadAndTobago"] = "tt";
        Flag["Tunisia"] = "tn";
        Flag["Turkey"] = "tr";
        Flag["Tuvalu"] = "tv";
        Flag["Tanzania"] = "tz";
        Flag["Uganda"] = "ug";
        Flag["Ukraine"] = "ua";
        Flag["Uruguay"] = "uy";
        Flag["UnitedStates"] = "us";
        Flag["Uzbekistan"] = "uz";
        Flag["VaticanCity"] = "va";
        Flag["SaintVincentAndTheGrenadines"] = "vc";
        Flag["Venezuela"] = "ve";
        Flag["Vietnam"] = "vn";
        Flag["Vanuatu"] = "vu";
        Flag["Samoa"] = "ws";
        Flag["Yemen"] = "ye";
        Flag["SouthAfrica"] = "za";
        Flag["Zambia"] = "zm";
        Flag["Zimbabwe"] = "zw";
    })(exports.Flag || (exports.Flag = {}));

    exports.RelationshipType = void 0;
    (function (RelationshipType) {
        RelationshipType["OneToOne"] = "oneToOne";
        RelationshipType["ManyToOne"] = "manyToOne";
        RelationshipType["ManyToMany"] = "manyToMany";
        RelationshipType["OneToMany"] = "oneToMany";
    })(exports.RelationshipType || (exports.RelationshipType = {}));

    exports.RelationMutate = void 0;
    (function (RelationMutate) {
        RelationMutate["Cascade"] = "cascade";
        RelationMutate["Restrict"] = "restrict";
        RelationMutate["SetNull"] = "setNull";
    })(exports.RelationMutate || (exports.RelationMutate = {}));

    exports.IndexType = void 0;
    (function (IndexType) {
        IndexType["Key"] = "key";
        IndexType["Fulltext"] = "fulltext";
        IndexType["Unique"] = "unique";
        IndexType["Spatial"] = "spatial";
        IndexType["Array"] = "array";
    })(exports.IndexType || (exports.IndexType = {}));

    exports.Runtime = void 0;
    (function (Runtime) {
        Runtime["Node180"] = "node-18.0";
        Runtime["Php80"] = "php-8.0";
        Runtime["Ruby31"] = "ruby-3.1";
        Runtime["Python39"] = "python-3.9";
    })(exports.Runtime || (exports.Runtime = {}));

    exports.Method = void 0;
    (function (Method) {
        Method["GET"] = "GET";
        Method["POST"] = "POST";
        Method["PUT"] = "PUT";
        Method["PATCH"] = "PATCH";
        Method["DELETE"] = "DELETE";
        Method["OPTIONS"] = "OPTIONS";
    })(exports.Method || (exports.Method = {}));

    exports.Status = void 0;
    (function (Status) {
        Status["Draft"] = "draft";
        Status["Cancelled"] = "cancelled";
        Status["Processing"] = "processing";
    })(exports.Status || (exports.Status = {}));

    exports.Compression = void 0;
    (function (Compression) {
        Compression["None"] = "none";
        Compression["Gzip"] = "gzip";
        Compression["Zstd"] = "zstd";
    })(exports.Compression || (exports.Compression = {}));

    exports.ImageGravity = void 0;
    (function (ImageGravity) {
        ImageGravity["Center"] = "center";
        ImageGravity["TopLeft"] = "top-left";
        ImageGravity["Top"] = "top";
        ImageGravity["TopRight"] = "top-right";
        ImageGravity["Left"] = "left";
        ImageGravity["Right"] = "right";
        ImageGravity["BottomLeft"] = "bottom-left";
        ImageGravity["Bottom"] = "bottom";
        ImageGravity["BottomRight"] = "bottom-right";
    })(exports.ImageGravity || (exports.ImageGravity = {}));

    exports.ImageFormat = void 0;
    (function (ImageFormat) {
        ImageFormat["Jpg"] = "jpg";
        ImageFormat["Jpeg"] = "jpeg";
        ImageFormat["Gif"] = "gif";
        ImageFormat["Png"] = "png";
        ImageFormat["Webp"] = "webp";
    })(exports.ImageFormat || (exports.ImageFormat = {}));

    exports.PasswordVersion = void 0;
    (function (PasswordVersion) {
        PasswordVersion["Sha1"] = "sha1";
        PasswordVersion["Sha224"] = "sha224";
        PasswordVersion["Sha256"] = "sha256";
        PasswordVersion["Sha384"] = "sha384";
        PasswordVersion["Sha512224"] = "sha512/224";
        PasswordVersion["Sha512256"] = "sha512/256";
        PasswordVersion["Sha512"] = "sha512";
        PasswordVersion["Sha3224"] = "sha3-224";
        PasswordVersion["Sha3256"] = "sha3-256";
        PasswordVersion["Sha3384"] = "sha3-384";
        PasswordVersion["Sha3512"] = "sha3-512";
    })(exports.PasswordVersion || (exports.PasswordVersion = {}));

    exports.ProviderType = void 0;
    (function (ProviderType) {
        ProviderType["Email"] = "email";
        ProviderType["Sms"] = "sms";
        ProviderType["Push"] = "push";
    })(exports.ProviderType || (exports.ProviderType = {}));

    exports.Account = Account;
    exports.AppwriteException = AppwriteException;
    exports.Avatars = Avatars;
    exports.Client = Client;
    exports.Databases = Databases;
    exports.Functions = Functions;
    exports.Graphql = Graphql;
    exports.Health = Health;
    exports.ID = ID;
    exports.Locale = Locale;
    exports.Messaging = Messaging;
    exports.Permission = Permission;
    exports.Query = Query;
    exports.Role = Role;
    exports.Storage = Storage;
    exports.Teams = Teams;
    exports.Users = Users;

    Object.defineProperty(exports, '__esModule', { value: true });

})(this.Appwrite = this.Appwrite || {}, null, window);
